
/* ================================================================== */
/*		NOTE GENERATION UNIT: Mandelbrot                      */
/*			version 001                                   */
/* ================================================================== */

#include "../tsrc/mplay.h"
#include "../proteus.h"
#include "../canvas.h"

#define KEYCENT		64
#define MINKEY		29
#define MAXKEY		90
#define MINVEL		16
#define MAXVEL		120
#define MINDELTA	50
#define MAXDELTA	1000

#define LIMD_L		150
#define LIMD_H		3000

#define RTAB		((rand()&63)+24)

char Mode[128];
int Ixtab[NCHANS], Entab[NCHANS];

/* ================================================================== */


static int iter=0;

struct _m1par {
  float Lx,Ly,Rx,Ry;
  float Dx,Dy,Cr,Ci;
  float Mx,My;
  int   Ix,Iy,Scale;
} Mandpar;

struct _m1par MSH, MSH0;


void InitGen(void)
{
 float f;
 int h, i, k;

  memset (&Mandpar, 0, sizeof(struct _m1par));

#ifdef QRAM
  HOSTP_F(0,Mandpar.Lx);
  HOSTP_F(1,Mandpar.Ly);
  HOSTP_F(2,Mandpar.Rx);
  HOSTP_F(3,Mandpar.Ry);
  HOSTP_I(0,h);
#else
  Mandpar.Lx = HOSTP_F(0);
  Mandpar.Ly = HOSTP_F(1);
  Mandpar.Rx = HOSTP_F(2);
  Mandpar.Ry = HOSTP_F(3);
  h  = HOSTP_I(0);
#endif
  if (h<2) k=2; else k=h;
  Mandpar.Dx = (Mandpar.Rx-Mandpar.Lx)/k;
  Mandpar.Dy = (Mandpar.Ry-Mandpar.Ly)/k;
  Mandpar.Cr = Mandpar.Lx;
  Mandpar.Ci = Mandpar.Ly;

  Mandpar.Scale = 32;
  f = Mandpar.Lx * Mandpar.Ly;
  f1init(f);
  _memcpy(&k,&f,4);
  Rseed = k;
  srand(k);

  f = (Mandpar.Lx+Mandpar.Ly) * h * 10;
  if (f==0) f=256; else if (f<0) f = -f;
  while (f<250) f*=2;
  while (f>3000) f/=2;
  Btempo=f;

  MSH0 = Mandpar;  
  MSH = Mandpar;

  k = HOSTP_X(1);
  for (i=0; i<32; i++) {
    Ixtab[i] = rand()&CHANMASK;
    Mode[(rand()&63)+24] ^= 1;
    Entab[i] = k&1; k>>=1;
   }
  k = HOSTP_X(2);
  for (i=32; i<NCHANS; i++) {
    Ixtab[i] = rand()&CHANMASK;
    Mode[(rand()&63)+24] ^= 1;
    Entab[i] = k&1; k>>=1;
   }
}


void mandel(struct _m1par *M)
{
 double t,r2,i2;

  r2 = M->Mx * M->Mx;
  i2 = M->My * M->My;
  t = r2 - i2 + M->Cr;
  M->My = 2 * M->Mx * M->My + M->Ci;
  M->Mx = t;
  t=r2+i2;
  if (t>=2) {
    Mode[(rand()&63)+24] ^= 1;
    M->Mx=0; M->My=0;
    M->Cr += M->Dx;
    if (M->Cr > M->Rx) {
      M->Cr = M->Lx; M->Ci += M->Dy;
      if (M->Ci > M->Ry) M->Ci = M->Ly;
     }
    iter=0;
   }
  iter++;
  M->Ix = M->Mx * M->Scale;
  M->Iy = M->My * M->Scale;
}


unsigned inrange(int n, unsigned lb, unsigned hb)
{
  while (Mode[n]==0) n=(n+1)&127;
  if (n<lb) {
    n += (((lb-n)/12+1)*12);
  }
  else if (n>hb) {
    n -= (((n-hb)/12+1)*12);
  }
  return(n);
}


/* ================================================================== */


void GenLoop(void)
{
 int mxv, Ntb, Nofs;
 unsigned t0,t1,maxt;		/* microsec */
 NoteEvent N;
 struct _m1par *M;

  M = &Mandpar;
  mxv=0; N.pitch=0; Nofs=0;

  asm { ldtimer; ldc 6; shl; ldc 10000; sum; st t0; }
  maxt=t0;
  M->Mx=0; M->My=0;
  N.startvol=N.deltavol=0;
  N.startpress=N.deltapress=0;
  N.lock=0;

  for (;;) {
    mandel(M);
    N.pitch = M->Ix + KEYCENT + Nofs;

    N.timbre = Ntb = Ixtab[iter&CHANMASK]; /*(Ix+Iy+N.pitch)&CHANMASK;*/

    N.pitch = inrange(N.pitch,MINRANGE(Ntb),MAXRANGE(Ntb));

    N.intensity = sqrt(M->Ix*M->Ix+M->Iy*M->Iy)+MINVEL;
    if (N.intensity>MAXVEL) N.intensity=MAXVEL;
    if (N.intensity>mxv) mxv=N.intensity;

    N.duration = ((iter&3)*QUARTER) +
		 (((MAXKEY+2)-N.pitch)*10) +
		 ((mxv-N.intensity)*20);

    N.deltat = QUARTER+(M->Iy*16);
    if (N.deltat>MAXDELTA) N.deltat=MAXDELTA;
     else if (N.deltat<MINDELTA) N.deltat=MINDELTA;

    t1 = t0+(N.duration*TimeD);
    if (t1>maxt) maxt=t1;		/* with duration */

    if (Entab[Ntb])
      t0 = note(&N,t0);
    else
      t0 = nothing(&N,t0);

    if (t0>maxt) maxt=t0;		/* with deltat */

/*
    TimeD += 5*(M->Ix-M->Iy);
    if (TimeD<(Btempo/2)) TimeD=Btempo*1.5;
     else if (TimeD>(Btempo*2)) TimeD=Btempo*0.66;
*/

    if (HostCmd) {
      int i,k;
/* ================================================================== */
/*		host command handling				      */
/* ================================================================== */
      switch(HostCmd) {
	case 27:  return;
	case '.': Mandpar=MSH;
		  srand(Rseed);
		  break;
	case ',': MSH=Mandpar;
		  break;
	case '+': Nofs++; break;
	case '-': Nofs--; break;
	case '?': k = RTAB%12; if (k<2) k=12;
		  for (i=13; i<96; i+=k)
		    Mode[i] ^= 1;
		  break;
	case '/': for (i=13; i<96; i++)
		    Mode[i] ^= 1;
		  break;
	case 'R': for (k=0; k<100; k++) mandel(M);
		  break;
	case 'G': TimeD /= 4; if (TimeD<LIMD_L) TimeD=LIMD_L;
		  t0=ntempo(TimeD,t0);
		  break;
	case 'F': TimeD /= 1.5; if (TimeD<LIMD_L) TimeD=LIMD_L;
		  t0=ntempo(TimeD,t0);
		  break;
	case 'Q': for (k=0; k<NCHANS; k++) t0=allnoteoff(k,t0+5);
		  break;
	case 'D': TimeD *= 4; if (TimeD>LIMD_H) TimeD=LIMD_H;
		  t0=ntempo(TimeD,t0);
		  break;
	case 'S': TimeD *= 1.5; if (TimeD>LIMD_H) TimeD=LIMD_H;
		  t0=ntempo(TimeD,t0);
		  break;
	case 'Z': Mandpar=MSH0;
		  srand(Rseed);
	case 'A': TimeD = Btempo;
		  t0=ntempo(TimeD,t0);
		  break;
	default:  M->Mx=0; M->My=0;
	          M->Cr=M->Lx; M->Ci += M->Dy;
	          if (M->Ci > M->Ry) M->Ci = M->Ly;
		  iter=0;
	          TimeD=Btempo;
		  srand(Rseed);
		  break;
/* ================================================================== */
       }
      HostCmd=0;
     }
    asm { ld t0; ldc 6; shr; adc -100; tin; }

   }
}


/* ================================================================== */

