
/* ================================================================== */
/*		NOTE GENERATION UNIT: Martin's Hopalong               */
/*			version 017                                   */
/* ================================================================== */

#include "../tsrc/mplay.h"
#include "../proteus.h"
#include "martin.h"

#define KEYCENT		60
#define MINVEL		24
#define MIDVEL		56
#define MAXVEL		90
#define BLKTHR		(MIDVEL+16)
#define VELRANGE	(MIDVEL-MINVEL)
#define MINDELTA	5
#define MAXDELTA	2000

#define LIMD_L		300
#define LIMD_H		5000

#define RTAB		((Rand()&63)+24)

#define R16		150000.0
#define R32		82500.0

Mhop2 Martpar,MSH;
int   iter,icyc;

int Rptr,N16,N32;
unsigned Mtrans,Etb;

/* ================================================================== */

unsigned trill  (NoteEvent *N,unsigned t);
unsigned mtrill (NoteEvent *N,unsigned t);
unsigned scale  (NoteEvent *N,unsigned t);
unsigned pscale (NoteEvent *N,unsigned t);
unsigned mscale (NoteEvent *N,unsigned t);
unsigned fgroup (NoteEvent *N,unsigned t);
unsigned block  (NoteEvent *N,unsigned t);
unsigned melody (NoteEvent *N,unsigned t);
unsigned noplay (NoteEvent *N,unsigned t);

/* ================================================================== */

char Mode[128];

char Majacc[12] =
	{ 1,0,0,0,1,0,0,1,0,0,0,0 };
char Minacc[12] =
	{ 1,0,0,1,0,0,0,1,0,0,0,0 };

int  Tbtab[NCHANS];
int  Iatab[NCHANS];
unsigned Busy[NCHANS];
int  Nbuf[NCHANS];
unsigned MelBusy;

int Dbuf[NCHANS] =
	{
	  2,2,4,2,2,4,8,6,
	  2,4,4,2,2,1,2,8,
	  6,2,4,1,2,12,2,2,
	  8,6,2,4,4,2,1,4,

	  3,1,5,1,3,5,7,5,
	  1,5,3,3,1,2,1,9,
	  5,3,3,2,1,13,1,3,
	  7,7,1,5,3,3,1,5,
	};

/* ================================================================== */

unsigned CanPlay   = 0xffffffff;

unsigned CanBlock  = 0xaa90c06d;
unsigned CanTrill  = 0x06304923;
unsigned CanScale  = 0x883f49b3;
unsigned CanPscale = 0xa41f4133;
unsigned CanTune   = 0x9840d97e;
unsigned IsPiano   = 0x002f0200;

/* ================================================================== */


void premartin(int limit,int loops)
{
 Mhop2 *EM = &Martpar;
 Mhop2  M;
 int i;
 double t,u;

  M = *EM;
  M.meanx = M.meany = 0;
  M.maxx = M.maxy = 0;
  M.x = onefrand(-1,1);
  M.y = onefrand(-1,1);

  for (i=0; i<loops; i++) {
    t=M.b*M.x-M.c; if (t<0) t= -t;
    if (M.x<0) u=M.y+sqrt(t);
     else if (M.x>0) u=M.y-sqrt(t);
      else u=M.y;
    M.y = M.a-M.x;
    M.x = u;
    t = fabs(M.x); if (t>M.maxx) M.maxx=t;
    M.meanx += t;
    t = fabs(M.y); if (t>M.maxy) M.maxy=t;
    M.meany += t;
   }
  EM->meanx = M.meanx/loops;
  EM->meany = M.meany/loops;
  EM->maxx = M.maxx;
  EM->maxy = M.maxy;
  t = (M.maxx>M.maxy? M.maxx:M.maxy);
  EM->sc = limit/t;
}


void martin2(Mhop2 *M)
{
 double t,u;

  t=M->b*M->x-M->c; if (t<0) t= -t;
  if (M->x<0) u=M->y+sqrt(t);
   else if (M->x>0) u=M->y-sqrt(t);
    else u=M->y;
  t = M->a-M->x;
  asm { ldc 0; ld &t; stnl 0; }
  asm { ldc 0; ld &u; stnl 0; }
  M->y = t;
  M->x = u;

  M->aix = M->ix = (u-M->meanx)*M->sc;
  M->sg = (M->ix==0? 0:(M->ix<0? -1:1));
  M->aiy = M->iy = (t-M->meany)*M->sc;
  M->sg += (M->iy==0? 0:(M->iy<0? -1:1));

  if (M->ix<0) M->aix = -M->aix;
  if (M->iy<0) M->aiy = -M->aiy;
  M->sq = sqrt(u*u+t*t);
  M->isq = M->sg*sqrt(M->ix*M->ix+M->iy*M->iy);
}


/* ================================================================== */


unsigned (*FAtab[32])(NoteEvent *,unsigned) =
	{
	 trill,trill,fgroup,scale,
	 trill,trill,fgroup,scale,
	 pscale,mscale,fgroup,noplay,
	 fgroup,scale,melody,noplay,
	 melody,mtrill,melody,melody,
	 noplay,fgroup,note,mscale,
	 note,note,scale,pscale,
	 note,note,note,mtrill
	};


/* ================================================================== */


void InitGen(void)
{
 int i,k,r,t;
 float f;
 unsigned (*p)();

  Martpar.a = HOSTP_F(0);
  Martpar.b = HOSTP_F(1);
  Martpar.c = HOSTP_F(2);
  Martpar.x = 0;
  Martpar.y = 0;
  Martpar.sc = HOSTP_I(0);

  f = Martpar.a*Martpar.b*Martpar.c;
  f1init(f);
  Martpar.Fseed=Fseq;
  _memcpy(&k,&f,4);
  Martpar.Rseed=k;
  srand(k);
  Srand(k);

  if (f==0) f=LIMD_L; else if (f<0) f = -f;
  while (f>LIMD_H) f/=3;
  while (f<LIMD_L) f*=3;
  TimeD = Btempo =f;
  N16=R16/TimeD;
  N32=R32/TimeD;

  premartin(64,100);
  f1init(Martpar.Fseed);

  for (i=0; i<NCHANS; i++) {
    Iatab[i]=0;
    Dbuf[i]*=SIXTEENTH;
    Tbtab[i]=i;
   }
  for (i=0; i<NCHANS; i++) {
    r=Rand()&CHANMASK;
    do { k=Rand()&CHANMASK; } while (k==r);
    p=FAtab[r&31]; FAtab[r&31]=FAtab[k&31]; FAtab[k&31]=p;
    t=Dbuf[r];  Dbuf[r]=Dbuf[k];   Dbuf[k]=t;
    Iatab[i]=r&0x10;
   }
  for (k=0; k<NCHANS; k++) {
    r=onefrand(0,32);
    Nbuf[k]=57+(r%16);
   }
  MSH=Martpar;
  Rptr=0;
  Mtrans=0;

  k=HOSTP_I(1); if (k==0) k=1;
  for (i=0; i<128; i++) {
    Mode[i] = Params.Mtempl[(i*k)&0x7f];
   }
  _memcpy(Params.Mtempl,Mode,128);
  Martpar.Mvel=MIDVEL;

}


/* ================================================================== */


unsigned noplay(NoteEvent *N,unsigned t)
{
 unsigned t1;

  t1=N->deltat*TimeD;
  if (t1>500000) t1=500000;
  t1 += t;
  Busy[Etb]=t1;
  Etb++; Etb &= CHANMASK;
  return(t1);
}


unsigned trill (NoteEvent *N,unsigned t)
{
 int i,nt,ht,dv,iH,iL,Mel;
 unsigned t1;

  if (!(Bittab[Etb&31]&CanTrill)) return(noplay(N,t));
  if (N->pitch<40) return(noplay(N,t));

  Mel = (MelBusy>t);
  if (Busy[Etb]>t) return(note(N,t));
  iH = N->intensity / 2; if (iH<MINVEL) iH=MINVEL;
  iL = N->intensity / 3; if (iL<MINVEL) iL=MINVEL;
  dv=Rand()&1;

  nt=(Rand()%9)+5;
  ht=Rand()%3;
  while (Mode[N->pitch+ht]==0) ht++;
  N->duration >>= (Mel? 3:2);
  if (N->duration>N16) N->duration=N16;
   else if (N->duration<N32) N->duration=N32;
  N->deltat = N->duration;

  t1=t+1;
  for (i=0; i<nt; i++) {
    N->intensity=iH;
    t = note(N,t);
    N->pitch += ht;
    N->intensity=iL;
    t = note(N,t);
    N->pitch -= ht;
    if (dv) iH-=2; else iH++; if (iH<MINVEL) iH=MINVEL;
   }
  Busy[Etb]=t;
  Martpar.Mvel-=3;
  return(t1);
}


unsigned mtrill (NoteEvent *N,unsigned t)
{
 int i,nt,ht,dv,rh,Mel;
 unsigned t1;

  if (!(Bittab[Etb&31]&CanTrill)) return(noplay(N,t));
  if (N->pitch<36) return(noplay(N,t));

  Mel = (MelBusy>t);
  if (Busy[Etb]>t) return(note(N,t));
  N->intensity /= 2;
  if (N->intensity<MINVEL) N->intensity=MINVEL;

  dv=Rand()&1;
  rh=Martpar.Rseed;
  nt=(Rand()&15)+12;
  ht=Rand()%3;

  N->duration /= (Mel? 6:5);
  if (N->duration>N16) N->duration=N16;
   else if (N->duration<N32) N->duration=N32;
  N->deltat = N->duration;

  t1=t+1;
  for (i=0; i<nt; i++) {
    if (rh&1) t = note(N,t); else t+=N->deltat*TimeD;
    if (rh&2) t = note(N,t); else t+=N->deltat*TimeD;
    rh = (rh>>2)|((rh&3)<<30);
    if (dv) N->intensity-=2; else N->intensity++;
    if (N->intensity<MINVEL) N->intensity=MINVEL;
   }
  Martpar.Mvel-=3;
  return(t1);
}


unsigned scale (NoteEvent *N,unsigned t)
{
 int i,k,nt,du,sgn;
 unsigned t1;

  sgn = Rand()&1;
  nt=4+(Rand()%7);
  k=Rand()&7;
  if (!(Bittab[Etb&31]&CanScale)) du=N->duration*2; else du=N->duration/2;
  N->deltat = du;
  N->duration = du/2;

  t1=t+1;
  for (i=0; i<nt; i++) {
    if (sgn) {
      t = note(N,t);
      N->pitch += ((Rand()%3)+1);
      if (i&1) { while (Mode[N->pitch]==0) N->pitch++; }
      N->intensity--;
     }
    else {
      t = note(N,t);
      N->pitch -= ((Rand()%3)+1);
      if (i&1) { while (Mode[N->pitch]==0) N->pitch--; }
      N->intensity--;
     }
    while (N->pitch<MINRANGE(Etb)) N->pitch+=12;
    while (N->pitch>MAXRANGE(Etb)) N->pitch-=12;
   }
  return(t1);
}


static unsigned HscT;


unsigned pscale (NoteEvent *N,unsigned t)
{
 int i,nt,np;
 unsigned t1;

  HscT=t;
  if (!(Bittab[Etb&31]&CanPscale)) return(noplay(N,t));
  nt=4+(Rand()%11);
  N->deltat = N->duration;
  N->duration -= 10;
  np = (int)N->pitch-36;
  while (np<MINRANGE(Etb)) np+=12;

  t1=t+1;
  for (i=0; i<nt; i++) {
    np += onefrand(-4,7);
    while (Mode[np]==0) np++;
    if ((np>MAXRANGE(Etb))||(np<MINRANGE(Etb))) return(t1);
    N->pitch=np;
    t = note(N,t);
    N->intensity--;
   }
  HscT=t;
  return(t1);
}


unsigned mscale (NoteEvent *N,unsigned t)
{
 int i,nt,np;
 unsigned t1;

  HscT=t;
  if (!(Bittab[Etb&31]&CanPscale)) return(noplay(N,t));
  nt=4+(Rand()%11);
  N->deltat = N->duration;
  N->duration -= 10;
  np = (int)N->pitch+36;
  while (np>MAXRANGE(Etb)) np-=12;

  t1=t+1;
  for (i=0; i<nt; i++) {
    np += onefrand(-7,4);
    while (Mode[np]==0) np--;
    if ((np>MAXRANGE(Etb))||(np<MINRANGE(Etb))) return(t1);
    N->pitch=np;
    t = note(N,t);
    N->intensity--;
   }
  HscT=t;
  return(t1);
}


unsigned fgroup (NoteEvent *N,unsigned t)
{
 int i,nt,np,du,lastP;
 unsigned k,t1;

  if (!(Bittab[Etb&31]&CanPscale)) return(noplay(N,t));
  if (Busy[Etb]>t) return(note(N,t));
  nt=4+(Rand()%15);
  np=N->pitch;

  du = N->duration/4;
  if (du<N16) du=SIXTEENTH;
  lastP=0;
  t1=t+1;

  for (i=0; i<nt; i++) {
    N->pitch = np+onefrand(-15,15);
    k = (N->pitch/2)&3;
    switch(k) {
      case 0: while (Mode[N->pitch]==0) N->pitch++;
	      N->deltat = du;
	      N->duration = du+5;
	      break;
      case 1: while (Mode[N->pitch]==0) N->pitch--;
	      N->deltat = du*2;
	      N->duration = (du*2)+5;
	      t1=t;
	      break;
      case 2: while (Mode[N->pitch]==0) N->pitch++;
	      N->deltat = du/2;
	      N->duration = (du/2)+5;
	      break;
      case 3: while (Mode[N->pitch]==0) N->pitch--;
	      N->deltat = du;
	      N->duration = du+5;
	      break;
     }
    if (N->pitch!=lastP) t = note(N,t);
     else t += (N->deltat*TimeD);
    lastP=N->pitch;
    Busy[Etb]=t;
   }
  return(t1);
}


unsigned block (NoteEvent *N,unsigned t)
{
 int i,k,np,is,mxd;
 unsigned t1;
 int durs[NCHANS];

  k=rand()&CanBlock;
  np=N->pitch+12;
  is=N->intensity;

  mxd=0;
  for (i=0; i<NCHANS; i++) {
    durs[i]=((Rand()%11)+1)*EIGHTH;
    if (durs[i]>mxd) mxd=durs[i];
   }
  N->deltat=EIGHTH+(mxd/4);

  for (i=0; i<NCHANS; i++) {
    N->timbre=i;
    N->duration=durs[i];
    if (k&1) {
      if (Rand()&1) {
	N->pitch=np+onefrand(-21,-3);
	while (Mode[N->pitch]==0) N->pitch++;
       }
      else {
	N->pitch=np+onefrand(3,21);
	while (Mode[N->pitch]==0) N->pitch--;
       }
      while (N->pitch<MINRANGE(i)) N->pitch+=12;
      while (N->pitch>MAXRANGE(i)) N->pitch-=12;
      N->intensity=is+onefrand(-10,2);
      t1=note(N,t); t+=5;
     }
    k>>=1;
   }
  Martpar.Mvel+=3;
  return(t1);
}


unsigned melody (NoteEvent *N,unsigned t)
{
 int i,nt,np,du,xm,xt,rv,is,bu;
 unsigned t1;

  if (MelBusy>t) return(t);
  if (Busy[Etb]>t) return(note(N,t));
  if (!(Bittab[Etb&31]&CanTune)) return(mtrill(N,t));
  xm=Mtrans;
  xt=(Rand()&3)+1;
  is=N->intensity+((2-xt)*4);
  if (is<MINVEL) is=MINVEL;
   else if (is>MIDVEL) is=MIDVEL;
  Martpar.Mvel = ((Martpar.Mvel*3)+is)/4;
  rv=Rand()&7;
  nt=7+(Rand()%11);
  bu=(Rand()%nt)/2;
  np=N->pitch;
  t1=t;

  for (i=0; i<nt; i++) {
    N->pitch = Nbuf[Rptr]+xm;
    du=Dbuf[Rptr]*xt;
    if (du>(2*WHOLE)) du=2*WHOLE;
    N->duration=du+10;
    N->deltat=du;
    N->intensity=is;
    if (rv&1) {
      if (Rptr==0) Rptr=NCHANS; Rptr--; is++;
     }
    else {
      Rptr++; if (Rptr==NCHANS) Rptr=0; is--;
     }
    if (du>=QUARTER) { while (Mode[N->pitch]==0) N->pitch++; }
    while (N->pitch<MINRANGE(Etb)) N->pitch+=12;
    while (N->pitch>MAXRANGE(Etb)) N->pitch-=12;
    MelBusy = t = note(N,t);
    if (i==bu) Busy[Etb]=t1=t;
   }
  return(t1);
}


/* ================================================================== */


void GenLoop(void)
{
 int i,k,Nofs,Cptr,Mel,u1,u2;
 unsigned t0,t1,Ntb,Rtb,oPitch;
 Mhop2 *M = &Martpar;
 NoteEvent N,N1;

  N.pitch=0;
  N.lock=N1.lock=0;

  Nofs=(Rand()%12)-6;
  M->Tofs = 0;
  for (i=0; i<NCHANS; i++) Busy[i]=0;
  MelBusy=Mel=0;
  for (i=0; i<4; i++) martin2(M);
  u1=u2=0;

  asm { ldtimer; ldc 6; shl; ldc 100000; sum; st t0; }
  t0=ntempo(TimeD,t0);
  iter=0; icyc=0; Cptr=0; oPitch=0;

  for (i=0; i<32; i++) {
    if (Iatab[i]==0x10)
      t0 = patch(i,i|0x10,t0);
   }

  for (;;) {

    martin2(M);

    Rtb = (M->Tofs+(oPitch&0x33))&CHANMASK;
    k = ((Rtb>39)&&(Rtb<48)? Rtb-8:Rtb);
    N.timbre = Etb = Ntb = Tbtab[k];

    N.pitch = ((int)M->sq+Nofs+KEYCENT)&127;

    if (Mel) N.duration = EIGHTH;
     else {
       N.duration = (((M->aix-Rtb)&7)+1)*EIGHTH;
       if (N.duration>(3*WHOLE)) N.duration=3*WHOLE;
      }
    N.deltat = HALF;

    martin2(M);

    k = ((M->ix*M->iy)%VELRANGE)+MIDVEL;
    M->Mvel = ((M->Mvel*3)+k)/4;
    if (M->Mvel<MINVEL) M->Mvel=MINVEL;
     else if (M->Mvel>MAXVEL) M->Mvel=MAXVEL;

    N.intensity=(Mel? M->Mvel*3/5:M->Mvel);

    while (Mode[N.pitch&0x7f]==0) N.pitch++;
    while (N.pitch<MINRANGE(Ntb)) N.pitch+=12;
    while (N.pitch>MAXRANGE(Ntb)) N.pitch-=12;
    if ((Mel==0)&&(N.pitch<36)) N.duration*=3;

    oPitch = N.pitch;

    N1=N;
    k=M->aix>>2;

    if (M->Mvel>BLKTHR) k|=4; else k&=3;

    if (k&1) {
      N1.pitch = Nbuf[Rptr++]; Rptr &= CHANMASK;
      while (Mode[N1.pitch&0x7f]==0) N1.pitch--;
      while (N1.pitch<MINRANGE(Ntb)) N1.pitch+=3;
      while (N1.pitch>MAXRANGE(Ntb)) N1.pitch-=3;
      note(&N1,t0); t0+=5;
     }
    if ((Mel==0)&&(k&2)) {
      N1.pitch = Nbuf[Rptr++]; Rptr &= CHANMASK;
      while (Mode[N1.pitch&0x7f]==0) N1.pitch++;
      while (N1.pitch<MINRANGE(Ntb)) N1.pitch+=4;
      while (N1.pitch>MAXRANGE(Ntb)) N1.pitch-=4;
      note(&N1,t0); t0+=5;
     }

    if (k&4) t1=block(&N,t0);

    if (((k&4)==0)||(t0==t1)) {
      k=M->aiy%((k%23)+2);

/* ---------------------------- */
      t1 = (FAtab[k&31])(&N,t0);
/* ---------------------------- */
     }

    if (t1==t0) t0+=(WHOLE*TimeD);
     else t0=t1;

    Mel = (MelBusy>t0);

    iter++;
    icyc+=24;

    if (u1||((iter%73)==0)) {
      if (Mel) u1=1;
       else {
	Martpar=MSH;
	Srand(Martpar.Rseed);
	f1init(Martpar.Fseed);
	Mode[RTAB] ^= 1;
	u1=0;
       }
     }

    if (u2||(icyc>Dbuf[Cptr])) {
      if (Mel) u2=1;
      else {
	k = Nbuf[Cptr++]%12;
	Cptr &= CHANMASK;
	for (i=0; i<k; i++)
	  Mode[i+96] = Mode[i+12];
	for (i=12; i<96; i++)
	  Mode[i] = Mode[i+k];
	Mtrans = (Nbuf[Rptr]%12)-6;
	M->Tofs = (M->Tofs+Mtrans)&CHANMASK;
	icyc = 0;
	k=Rand()&CHANMASK;
	Iatab[k]^=0x10;
	t0 = patch(k,(k&15)+Iatab[k],t0);
	MSH=Martpar;
	Martpar.Fseed=Fseq;
	Martpar.Rseed=Rseed;
	TimeD = Btempo;
	u2=0;
       }
     }

    if (HostCmd) {
      switch(HostCmd) {
	case '.': Martpar=MSH;
		  Srand(Martpar.Rseed);
		  f1init(Martpar.Fseed);
		  break;
	case ',': MSH=Martpar;
		  Martpar.Fseed=Fseq;
		  Martpar.Rseed=Rseed;
		  break;
	case '>': M->Mvel = (M->Mvel*5)/4;
		  if (M->Mvel>MAXVEL) M->Mvel=MAXVEL;
		  break;
	case '<': M->Mvel = (M->Mvel*3)/4;
		  if (M->Mvel<MINVEL) M->Mvel=MINVEL;
		  break;
	case '?': k = (N.pitch+7)%12;
		  for (i=12; i<96; i+=12)
		    Mode[i+k] ^= 1;
		  break;
	case '+': Nofs+=12; break;
	case '-': Nofs-=12; break;
	case 'E': for (i=0; i<16; i++) {
		    k=Tbtab[i]; Tbtab[i]=Tbtab[(NCHANS-1)-i];
		    Tbtab[(NCHANS-1)-i]=k;
		  }
		  break;
	case 'R': for (k=0; k<100; k++) martin2(M);
		  break;
	case 'F': TimeD /= 1.5; if (TimeD<LIMD_L) TimeD=LIMD_L;
		  N16=R16/TimeD;
		  N32=R32/TimeD;
		  t0=ntempo(TimeD,t0);
		  break;
	case 'D': TimeD *= 4; if (TimeD>LIMD_H) TimeD=LIMD_H;
		  N16=R16/TimeD;
		  N32=R32/TimeD;
		  t0=ntempo(TimeD,t0);
		  break;
	case 'G': TimeD /= 4; if (TimeD<LIMD_L) TimeD=LIMD_L;
		  N16=R16/TimeD;
		  N32=R32/TimeD;
		  t0=ntempo(TimeD,t0);
		  break;
	case 'S': TimeD *= 1.5; if (TimeD>LIMD_H) TimeD=LIMD_H;
		  N16=R16/TimeD;
		  N32=R32/TimeD;
		  t0=ntempo(TimeD,t0);
		  break;
	case 'Q': for (k=0; k<NCHANS; k++) t0=allnoteoff(k,t0+5);
		  break;
	case 'Z': M->x=0; M->y=0;
		  Srand(Martpar.Rseed);
		  f1init(Martpar.Fseed);
		  _memcpy(Mode,Params.Mtempl,128);
		  Martpar.Mvel=MIDVEL;
		  Nofs=0;
	case 'A': TimeD = Btempo;
		  N16=R16/TimeD;
		  N32=R32/TimeD;
		  t0=ntempo(TimeD,t0);
		  M->Mvel=MIDVEL;
		  break;
	default:  Martpar=MSH;
		  M->x = HostCmd/38.0;
		  M->y = HostCmd/47.0;
		  M->Tofs = (HostCmd-' ')&CHANMASK;
		  Srand(Martpar.Rseed);
		  f1init(Martpar.Fseed);
#if (0)
		  k=0;
		  for (i=M->Tofs; i<84; i++) {
		    Mode[i] ^= Majacc[k++];
		    if (k==12) k=0;
		   }
#endif
		  break;
       }
      HostCmd=0;
     }

    TimeD += onefrand(-10,10);

    if (iter&32) t0 += (iter&3)<<20;

    asm { ld t0; ldc 6; shr; adc -100; tin; }

   }
}


/* ================================================================== */

