
/* ================================================================== */
/*		NOTE GENERATION UNIT: Martin's Hopalong               */
/*			version 005a                                  */
/* ================================================================== */

#include "../tsrc/mplay.h"
#include "../proteus.h"
#include "../moc1.h"
#include "martin.h"

#define KEYCENT		60
#define MINKEY		28
#define MAXKEY		90
#define MINVEL		44
#define MIDVEL		58
#define MAXVEL		80
#define MINVOL		32
#define MIDVOL		58
#define MAXVOL		90
#define MINDELTA	5
#define MAXDELTA	2000

#define LIMD_L		150
#define LIMD_H		3000

#define RTAB		((rand()&63)+24)

#define mrand()		martin(&Mpar2)

Mhop  Martpar,Mpar2,MSH,MSH0;
int   N16, N32;

int  Entab[NCHANS], Busy[NCHANS];

char Mode[128];
int  Epitch[NCHANS];

int Section, BaseMsc, Pizzi, EnVol;

int Epsc1[12] = 
	{ 1,1,2,4,4,5,7,7,8,8,10,11 };
int Epsc2[12] = 
	{ 0,1,2,2,4,5,6,9,9,10,11,11 };

/* ================================================================== */

unsigned trill  (NoteEvent *N,unsigned t);
unsigned mtrill (NoteEvent *N,unsigned t);
unsigned scale  (NoteEvent *N,unsigned t);
unsigned pscale (NoteEvent *N,unsigned t);
unsigned mscale (NoteEvent *N,unsigned t);
unsigned fgroup (NoteEvent *N,unsigned t);
unsigned block  (NoteEvent *N,unsigned t);
unsigned melody (NoteEvent *N,unsigned t);
unsigned noplay (NoteEvent *N,unsigned t);

/* ================================================================== */
/*		FORMULA: Martin's Hopalong			      */
/* ================================================================== */

int martin(Mhop *M)
{
 double t,u;

/* iterazione algoritmo */
  t=M->b*M->x-M->c; if (t<0) t= -t;
  if (M->x<0) u=M->y+sqrt(t);
   else if (M->x>0) u=M->y-sqrt(t);
    else u=M->y;
  M->y = M->a-M->x;
  M->x = u;

  M->ix = cos(M->x)*256;
  M->iy = sin(M->y)*256;

  return (M->ix*M->iy);
}


int nextp(int n)
{
  char c = Mode[n&127];
  return (c-8);
}

int nextn(int i, int n)
{
  if (i&1)
    return (((n/12)*12) + Epsc2[n%12]);
  else
    return (((n/12)*12) + Epsc1[n%12]);
}


/* ================================================================== */
/*		GENERATOR loop					      */
/*								      */
/*	Host parameter use:					      */
/*	F0 F1 F2		a,b,c				      */
/*	I0			scale factor			      */
/*	I1			mode rotation			      */
/*								      */
/* ================================================================== */

void InitGen(void)
{
 int i,k;
 FLOAT f;

  Martpar.a = HOSTP_F(0);
  Martpar.b = HOSTP_F(1);
  Martpar.c = HOSTP_F(2);
  Martpar.x = Martpar.y = HOSTP_F(3);
  BaseMsc = HOSTP_I(0); if (BaseMsc==0) BaseMsc=32;

  f = Martpar.a*Martpar.b*Martpar.c;
  f1init(f);
  _memcpy(&k,&f,4);
  Rseed=k;
  srand(k);

/* Btempo unit = 500-6000 us */
/*
  if (f==0) f=500;
   else if (f<0) f = -f;
  while (f<1000) f*=3;
  while (f>6000) f/=3;
*/
  f = 1000;
  Btempo=f;
  N16=200000.0/Btempo;
  N32=100000.0/Btempo;

  k = HOSTP_X(1);
  for (i=0; i<32; i++) 
  	{ Entab[i]=k&1; k>>=1; }
  k = HOSTP_X(2);
  for (i=32; i<NCHANS; i++) 
  	{ Entab[i]=k&1; k>>=1; }
  MSH0=Martpar;
  MSH=Martpar;
  Mpar2=Martpar;

  for (i=0; i<NCHANS; i++) {
  	Busy[i]=0;
  	Epitch[i]=KEYCENT;
  }

  k = HOSTP_X(3);
  Section = (k==0? 2:(k&7));
}


unsigned inrange(int n, unsigned lb, unsigned hb)
{
  if (n<lb) {
    n += (((lb-n)/12+1)*12);
  }
  else if (n>hb) {
    n -= (((n-hb)/12+1)*12);
  }
  return(n);
}


/* ================================================================== */


unsigned vnote (NoteEvent *N, unsigned t)
{
  static unsigned t1=0, p0=61, mv=64, opz=0;
  int k;

  mv = (mv + N->intensity) / 2;
  k = (mv+N->pitch+t) & 31;
  if (N->timbre==0) {
    if ((N->pitch!=48) && (N->pitch!=55) && (N->pitch!=62) && (N->pitch!=69)) {
      if (t<t1) return t;
      t1 = chknote(N,t) - N16;
      if (N->pitch > p0)
       { while ((int)(N->pitch - p0) > 7) N->pitch -= 7; }
      else
       { while ((int)(p0 - N->pitch) > 7) N->pitch += 7; }
      if (N->pitch<48) N->pitch = 48+(t&7);
       else if (N->pitch>84) N->pitch = 84-(t&7);
    }
    p0 = N->pitch;
    N->intensity = mv;
    if (Pizzi && (opz==0)) {
      patch(0,72,t); opz=1;
    }
    else if ((Pizzi==0) && opz) {
      patch(0,(k&3? 65:75),t); opz=0;
    }
  }
  if (k) {
    if (k<16) {
      pressure(N->timbre,(t&63)|64,t);
    }
    else if (EnVol) {
      int v;
      v = mv + nextp(t);
      if (v<MINVOL) v=MINVOL; else if (v>MAXVOL) v=MAXVOL;
      volume(N->timbre,v,t);
      EnVol = 0;
    }
  }
  return note(N,t);
}


/* ================================================================== */


unsigned noplay(NoteEvent *N,unsigned t)
{
 unsigned t1;
 int Etb = N->timbre;

  t1=N->deltat*TimeD;
  if (t1>500000) t1=500000;
  t1 += t;
  Busy[Etb]=t1;
  Etb++; Etb &= CHANMASK;
  return(t1);
}


unsigned trill (NoteEvent *N,unsigned t)
{
 int i,nt,ht,dv,iH,Mel;
 unsigned t1;

  if (N->pitch<40) return(noplay(N,t));

  Mel = 1; Pizzi = 0; EnVol = 0;
  iH = N->intensity / 2; if (iH<MINVEL) iH=MINVEL;
  dv=mrand()&1;

  nt=(abs(mrand())%9)+12;
  ht=abs(mrand())%2+1;
  N->duration >>= (Mel? 3:2);
  if (N->duration>N32) N->duration=N32;
  N->deltat = N->duration+1;

  t1=t+1;
  for (i=0; i<nt; i++) {
    N->intensity=iH;
    t = vnote(N,t);
    N->pitch += ht;
    t = vnote(N,t);
    N->pitch -= ht;
    if (dv) iH-=2; else iH++; if (iH<MINVEL) iH=MINVEL;
   }
  return(t1);
}


unsigned mtrill (NoteEvent *N,unsigned t)
{
 int i,nt,ht,dv,rh,Mel;
 unsigned t1;

  if (N->pitch<36) return(noplay(N,t));

  Mel = 1; Pizzi = 0; EnVol = 0;

  dv=mrand()&1;
  rh=Rseed;
  nt=(mrand()&15)+6;
  ht=abs(mrand())%3+1;

  N->duration /= (Mel? 6:5);
  if (N->duration>N32) N->duration=N32;
  N->deltat = N->duration+1;

  t1=t+1;
  for (i=0; i<nt; i++) {
    if (rh&1) t = vnote(N,t); else t+=N->deltat*TimeD;
    if (rh&2) t = vnote(N,t); else t+=N->deltat*TimeD;
    rh = (rh>>2)|((rh&3)<<30);
    if (dv) N->intensity-=2; else N->intensity++;
    if (N->intensity<MINVEL) N->intensity=MINVEL;
   }
  return(t1);
}


unsigned scale (NoteEvent *N,unsigned t)
{
 int i,k,nt,du,sgn;
 unsigned t1;

  sgn = mrand()&1;
  nt=4+(abs(mrand())%7);
  k=mrand()&7;
  N->deltat = abs(du);
  N->duration = abs(du/2);
  if (N->duration>N16) N->duration=N16;
  Pizzi = (nt==2); EnVol = 0;

  t1=t+1;
  for (i=0; i<nt; i++) {
    if (sgn) {
      t = vnote(N,t);
      N->pitch += ((abs(mrand())%3)+1);
      if (i&1) N->pitch = nextn(N->timbre,N->pitch+1);
      N->intensity--;
     }
    else {
      t = vnote(N,t);
      N->pitch -= ((abs(mrand())%3)+1);
      if (i&1) N->pitch = nextn(N->timbre,N->pitch-1);
      N->intensity--;
     }
    N->pitch = inrange(N->pitch,MINRANGE(N->timbre),MAXRANGE(N->timbre));
   }
  return(t1);
}


static unsigned HscT;


unsigned pscale (NoteEvent *N,unsigned t)
{
 int i,nt,np;
 unsigned t1;

  HscT=t;
  nt=4+(abs(mrand())%11);
  N->duration -= 10;
  if (N->duration>N16) N->duration=N16;
  N->deltat = N->duration + 10;
  Pizzi = (nt==2); EnVol = 0;
  np = Epitch[N->timbre]+nextp((int)N->pitch);

  t1=t+1;
  for (i=0; i<nt; i++) {
    np += onefrand(-2,7);
    N->pitch = nextn(N->timbre,inrange(np,MINRANGE(N->timbre),MAXRANGE(N->timbre)));
    t = vnote(N,t);
    N->intensity--;
   }
  HscT=t;
  return(t1);
}


unsigned mscale (NoteEvent *N,unsigned t)
{
 int i,nt,np;
 unsigned t1;

  HscT=t;
  nt=4+(abs(mrand())%11);
  N->duration -= 10;
  if (N->duration>N16) N->duration=N16;
  N->deltat = N->duration + 10;
  Pizzi = (nt==2); EnVol = 1;
  np = Epitch[N->timbre]+nextp((int)N->pitch);

  t1=t+1;
  for (i=0; i<nt; i++) {
    np += onefrand(-7,2);
    N->pitch = nextn(N->timbre,inrange(np,MINRANGE(N->timbre),MAXRANGE(N->timbre)));
    t = vnote(N,t);
    N->intensity--;
   }
  HscT=t;
  return(t1);
}


unsigned fgroup (NoteEvent *N,unsigned t)
{
 int i,nt,np,du,is,lastP;
 unsigned k,t1;
 int Etb = N->timbre;

  if ((Etb>39)&&(Etb<48)) return(t+5);
  nt=4+(abs(mrand())%7);
  np=N->pitch;
  Pizzi = (t&1); EnVol = 1;

  du = abs(N->duration/5);
  if (N->duration>N16) N->duration=N16;
  lastP=0;
  t1=t+10;

  is = (MINVEL+du)&127;
  if (is<MIDVEL) is=MIDVEL;
   else if (is>MAXVEL) is=MAXVEL;

  for (i=0; i<nt; i++) {
    N->pitch = nextn(i,np+onefrand(-15,15));
    k = (N->pitch/2)&3;
    switch(k) {
      case 0: N->pitch++;
	      N->deltat = du+5;
	      N->duration = du;
	      break;
      case 1: N->pitch--;
	      N->deltat = (du*2)+5;
	      N->duration = du*2;
	      t1=t;
	      break;
      case 2: N->pitch++;
	      N->deltat = (du/2)+5;
	      N->duration = du/2;
	      break;
      case 3: N->pitch--;
	      N->deltat = du+5;
	      N->duration = du;
	      break;
     }
    N->pitch = inrange(N->pitch,MINRANGE(N->timbre),MAXRANGE(N->timbre));
    if (N->pitch!=lastP) t = vnote(N,t);
    lastP=N->pitch;
    Busy[Etb]=t;
   }
  return(t1);
}


unsigned block (NoteEvent *N,unsigned t)
{
 int i,k,np,is,mxd;
 unsigned t1,dt;
 int durs[NCHANS];

  np=N->pitch+12;
  is=(~N->intensity)&63;

  mxd=0;
  for (i=0; i<NCHANS; i++) {
    durs[i]=((abs(mrand())%3)+1)*N16;
    if (durs[i]>mxd) mxd=durs[i];
   }
  if (Section==3) mxd /= 2;
  N->deltat=mxd*2+N16;

  k=rand();
  dt = 0;
  for (i=1; i<32; i++) {
    N->timbre=i;
    N->duration=durs[i];
    if (k&1) {
      if (mrand()&1) {
	N->pitch=nextn(i,np+onefrand(-21,-3));
       }
      else {
	N->pitch=nextn(i,np+onefrand(3,21));
       }
      N->pitch = inrange(N->pitch,MINRANGE(N->timbre),MAXRANGE(N->timbre));
      N->intensity=is+onefrand(-10,2);
      t1=note(N,t+dt); t+=10;
     }
    k>>=1;
    if (Section==6) dt=durs[31-i]/2; else dt=durs[mrand()&31];
   }
  k=rand() & 0x7f;
  for (i=32; i<39; i++) {
    N->timbre=i;
    N->duration=durs[i];
    if (k&1) {
      if (mrand()&1) {
	N->pitch=nextn(i,np+onefrand(-21,-3));
       }
      else {
	N->pitch=nextn(i,np+onefrand(3,21));
       }
      N->pitch = inrange(N->pitch,MINRANGE(N->timbre),MAXRANGE(N->timbre));
      N->intensity=is+onefrand(-10,2);
      t1=note(N,t+dt); t+=10;
     }
    k>>=1;
    if (Section==6) dt=durs[mrand()&31+32]; else dt=durs[(NCHANS-1)-i]/2;
   }
  return(t1);
}


unsigned melody (NoteEvent *N,unsigned t)
{
 int i,nt,np,du,xt,is;
 Mhop *M = &Martpar;
 NoteEvent N1;
 unsigned t1;
 int Etb = N->timbre;

  if ((Etb>39)&&(Etb<48)) return(t+10);
  N1=*N;
  xt=(mrand()&3)+1;
  is=N1.intensity+((2-xt)*4);
  if (is<MIDVEL) is=MIDVEL;
   else if (is>MAXVEL) is=MAXVEL;
  nt=7+(abs(mrand())%23);
  np=N1.pitch;
  Pizzi = 0; EnVol = 1;
  t1=t+10;

  for (i=0; i<nt; i++) {
    martin(M);
    N1.pitch = nextn(Etb,Epitch[Etb]+nextp(M->ix));
    while (N1.pitch>MAXRANGE(Etb)) N1.pitch-=12;
    du=(t&1? THIRTYSECOND/2:((M->iy&3)+1)*xt*THIRTYSECOND);
    if (du>WHOLE) du=WHOLE;
    N1.duration=du;
    N1.deltat=du;
    N1.intensity=(is+(2-xt)*i);
    if ((N1.intensity>MINVEL)&&(N1.intensity<MAXVEL)) {
      t = vnote(&N1,t);
      if (i<(nt/2)) t1=t;
    }
   }
  return(t1);
}


/* ================================================================== */


void GenLoop(void)
{
 int Ntb,i,k,iter,Tofs,Nofs,longnote,jctr;
 int rIt, rDt;
 unsigned t0, t1;
 Mhop *M = &Martpar;
 NoteEvent N,N1;

  N.startvol=N.deltavol=0;
  N.startpress=N.deltapress=0;
  N.pitch=0; Pizzi=0; EnVol = 1;

  for (i=0; i<4; i++) martin(M);
  Nofs=0;
  Tofs = mrand()&CHANMASK;

  asm { ldtimer; ldc 6; shl; ldc 100000; sum; st t0; }
  t0=ntempo(TimeD,t0);
  iter=1;
  jctr=0;

  for (;;) {

    martin(M);

    Ntb = (int)(M->ix & CHANMASK);
    if (!Entab[Ntb] || (Ntb>39)) Ntb=0;
    N.timbre = Ntb;

    N.pitch = Epitch[Ntb] + nextp(M->ix/2);
    N.pitch = nextn(Ntb,inrange(N.pitch,MINRANGE(Ntb),MAXRANGE(Ntb)));
    Epitch[Ntb] = N.pitch;

    oevent(M->ix,M->iy,t0);

    N.deltat = (M->ix&3)*EIGHTH;
    t1 = chknote(&N, t0);
    
    longnote = (M->iy < 0);
    if (longnote)
      N.duration = ((M->iy&7)+2)*HALF;
    else
      N.duration = ((M->iy&7)+2)*SIXTEENTH;
    if ((signed)N.duration <= 0) N.duration = QUARTER;

    if (longnote) {
      N.intensity = MINVEL;
    }
    else {
      N.intensity = (abs(M->iy)%(MIDVEL-MINVEL))+MINVEL;
      if (N.intensity<MINVEL) N.intensity=MINVEL;
       else if (N.intensity>MAXVEL) N.intensity=MAXVEL;
    }
    if (N.deltat < N.duration) N.deltat = N.duration+10;
    EnVol = 1;

    N1=N;
    N1.intensity -= 20;
    if (N1.intensity<MINVEL) N1.intensity=MINVEL;
     else if (N1.intensity>MAXVEL) N1.intensity=MAXVEL;

    if (Ntb==0) {
      switch (Mode[rand()]) {
        case 0: trill(&N,t0); break;
        case 1: scale(&N,t0); break;
        case 2: pscale(&N1,t0); break;
        case 3: mscale(&N,t0); break;
        case 4:
        case 5:
        case 6: fgroup(&N1,t0); break;
        default: t1 = melody(&N,t0); break;
      }
      t1 += N16;
    }
    else {
     switch (Section&15) {
      case 1:	    if ((mrand()&3)==0) {
		      N1.pitch = N.pitch+onefrand(-4,-1);
		      N1.lock = 0;
		      t1 = vnote(&N1,t0);
		     }
		    N.lock = 0;
		    if (Ntb) {
		      rIt = 70000+(N1.duration<<6)-((88-N1.pitch)<<8);	/* usec */
		      rDt = -(((mrand()&15)+1)<<9);
		      repnote(&N,rIt,rDt,t0);
		    }
		    if ((++jctr)>=12) { Section=Mode[mrand()]; jctr=0; }
		    break;

      case 2:	    if (Ntb && ((abs(mrand())%7)==1)) {
		      N1.pitch = N.pitch+onefrand(-9,-4);
		      N1.lock = 0;
		      rIt = 80000+(N1.duration<<5)-((88-N1.pitch)<<8);
		      rDt = ((mrand()&31)+1)<<9;
		      repnote(&N1,rIt,rDt,t0); t0+=10;
		     }
		    if (Ntb && ((abs(mrand())%3)==2)) {
		      N1.pitch = N.pitch+onefrand(4,9);
		      N1.lock = 0;
		      rIt = 120000+(N1.duration<<7)-((88-N1.pitch)<<8);
		      rDt = -(((mrand()&15)+1)<<9);
		      repnote(&N,rIt,rDt,t0); t0+=10;
		     }
		    N.lock = 0;
		    if ((++jctr)>=9) { Section=Mode[mrand()]; jctr=0; }
		    t1 = fgroup(&N1,t0);
		    break;

      case 3:	    N.lock = 0;
      		    N.deltat <<= 4;
		    t1 = block(&N,t0);
		    if ((++jctr)>=8) { Section=Mode[mrand()]; jctr=0; }
		    break;

      case 4:	    N.lock = 0;
		    melody(&N,t0);
		    if ((++jctr)>=8) { Section=Mode[mrand()]; jctr=0; }
		    break;

      case 5:	    N.lock = 0;
      		    switch ((Ntb+N.pitch)%5) {
		      case 0: trill(&N,t0); break;
		      case 1: scale(&N,t0); break;
		      case 2: pscale(&N,t0); break;
		      case 3: mscale(&N,t0); break;
		      case 4: t1 = fgroup(&N,t0); break;
		    }
		    if ((++jctr)>=16) { Section=Mode[mrand()]; jctr=0; }
		    break;

      case 6:	    N.lock = 0;
      		    switch ((Ntb-N.pitch)&3) {
		      case 0: trill(&N,t0); break;
		      case 1: mtrill(&N,t0); break;
		      case 2: t1 = block(&N,t0); break;
		      case 3: t1 = melody(&N,t0); break;
		    }
		    if ((++jctr)>=24) { Section=Mode[mrand()]; jctr=0; }
		    break;

      case 7:	    N.lock = 0;
		    t1 = noplay(&N,t0);
		    if ((++jctr)>=24) { Section=Mode[mrand()]; jctr=0; }
		    break;

      default:	    if (Ntb && ((rand()%3)==0)) {
		      N1.pitch = nextn(N.timbre,N.pitch+onefrand(-7,-3));
		      N1.lock = 0;
		      rIt = 150000+(N1.duration<<5)-((88-N1.pitch)<<8);
		      rDt = ((mrand()&31)+1)<<9;
		      t1 = repnote(&N1,rIt,rDt,t0); t0+=10;
		     }
		    if (Ntb && ((rand()%3)==1)) {
		      N1.pitch = nextn(N.timbre,N.pitch+onefrand(3,8));
		      N1.lock = 0;
		      rIt = 200000+(N1.duration<<6)-((88-N1.pitch)<<8);
		      rDt = -(((mrand()&31)+1))<<9;
		      t1 = repnote(&N1,rIt,rDt,t0); t0+=10;
		     }
		    N.lock = 0;
		    t1 = vnote(&N,t0);
		    break;
      }
     }
    iter++;

    if ((iter&0x1f)==(Ntb&31)) {
      for (k=0; k<NCHANS; k++) t0=allnoteoff(k,t0+5);
    }
    else if (HostCmd&0x80) {
/* ================================================================== */
/*		host command handling				      */
/* ================================================================== */
      switch(HostCmd&0x7f) {
	case 27:  return;
	case '.': Martpar=MSH;
		  srand(Rseed);
		  break;
	case ',': MSH=Martpar;
		  break;
	case '+': Nofs++; break;
	case '-': Nofs--; break;
	case 'R': for (k=0; k<100; k++) martin(M);
		  break;
	case 'G': TimeD /= 4; if (TimeD<LIMD_L) TimeD=LIMD_L;
		  t0=ntempo(TimeD,t0);
		  break;
	case 'F': TimeD /= 1.5; if (TimeD<LIMD_L) TimeD=LIMD_L;
		  t0=ntempo(TimeD,t0);
		  break;
	case 'Q': for (k=0; k<NCHANS; k++) t0=allnoteoff(k,t0+5);
		  break;
	case 'D': TimeD *= 4; if (TimeD>LIMD_H) TimeD=LIMD_H;
		  t0=ntempo(TimeD,t0);
		  break;
	case 'S': TimeD *= 1.5; if (TimeD>LIMD_H) TimeD=LIMD_H;
		  t0=ntempo(TimeD,t0);
		  break;
	case 'Z': Martpar=MSH0;
		  srand(Rseed);
	case 'A': TimeD = Btempo;
		  t0=ntempo(TimeD,t0);
		  break;
	case '0':
	case '1':
	case '2':
	case '3':
	case '4': Section=HostCmd&7; break;
	case '5':
	case '6':
	case '7': Section=HostCmd&7;
		  for (k=0; k<NCHANS; k++) t0=allnoteoff(k,t0+5);
	default:  M->x = HostCmd/128.0;
		  M->y = HostCmd/127.0;
		  Tofs = HostCmd&CHANMASK;
		  srand(Rseed);
		  break;
/* ================================================================== */
       }
      HostCmd=(iter&0x7f);
     }
    t0 = t1;
    asm { ld t1; ldc 6; shr; adc -100; tin; }

   }
}


/* ================================================================== */

