
/* ================================================================== */
/*		NOTE GENERATION UNIT: Martin's Hopalong               */
/*			version 005a                                  */
/* ================================================================== */

#include "../tsrc/mplay.h"
#include "../proteus.h"
#include "../moc1.h"
#include "martin.h"

#ifndef M_PI
#define M_PI        3.14159265358979323846	/* pi */
#endif
#ifndef M_PI_2
#define M_PI_2      1.57079632679489661923	/* pi/2 */
#endif

#define KEYCENT		62
#define MINKEY		28
#define MAXKEY		90
#define MINVEL		36
#define MIDVEL		68
#define MAXVEL		110
#define MINDELTA	5
#define MAXDELTA	2000

#define M_SCALE		128.0

#define LIMD_L		150
#define LIMD_H		3000

#define ISPIANO(i)	(((i)>=16)&&((i)<=19))

#define RTAB		((rand()&63)+24)
#define mrand()		(martin(&Mpar2))
#define amrand()	(abs(martin(&Mpar2)))

Mhop  Martpar,MSH,MSH0,Mpar2;
int   N16, N32;

unsigned Entab[NCHANS], Busy[NCHANS];

char Mode[256], Smode[128];

int Section, BaseMsc, SoloIns;


/* ================================================================== */

unsigned trill  (NoteEvent *N,unsigned t);
unsigned mtrill (NoteEvent *N,unsigned t);
unsigned scale  (NoteEvent *N,unsigned t);
unsigned pscale (NoteEvent *N,unsigned t);
unsigned mscale (NoteEvent *N,unsigned t);
unsigned fgroup (NoteEvent *N,unsigned t);
unsigned block  (NoteEvent *N,unsigned t);
unsigned melody (NoteEvent *N,unsigned t);
unsigned noplay (NoteEvent *N,unsigned t);

/* ================================================================== */
/*		FORMULA: Martin's Hopalong			      */
/* ================================================================== */

int martin(Mhop *M)
{
 FLOAT t,u;
 int f;

/* iterazione algoritmo */
  t=M->b*M->x-M->c; if (t<0) t= -t;
  if (M->x<0) u=M->y+sqrt(t);
   else if (M->x>0) u=M->y-sqrt(t);
    else u=M->y;
  M->y = M->a-M->x;
  M->x = u;

  M->ix = cos(M->x)*M_SCALE;
  M->iy = sin(M->y)*M_SCALE;
  f = M->ix*M->iy;
  M->sg = (f<0? -1: (f>0? 1:0));

  return(f);
}


/* ================================================================== */
/*		GENERATOR loop					      */
/*								      */
/*	Host parameter use:					      */
/*	F0 F1 F2		a,b,c				      */
/*	I0			scale factor			      */
/*	I1			mode rotation			      */
/*								      */
/* ================================================================== */

void InitGen(void)
{
 int i,j,k;
 FLOAT f;

  Martpar.a = HOSTP_F(0);
  Martpar.b = HOSTP_F(1);
  Martpar.c = HOSTP_F(2);
  Martpar.x = Martpar.y = HOSTP_F(3);
  Martpar.lx = 0;
  Martpar.hx = 0;
  Martpar.sc = 10;
  BaseMsc = HOSTP_I(0); if (BaseMsc==0) BaseMsc=32;

  f = Martpar.a*Martpar.b*Martpar.c;
  f1init(f);
  _memcpy(&k,&f,4);
  Rseed=k;
  srand(k);

/* Btempo unit = 500-6000 us */
/*  if (f==0) f=500;
   else if (f<0) f = -f;
  while (f<1000) f*=2;
  while (f>3000) f/=2;
  Btempo=f; */
  Btempo=2700;
  N16=80000.0/Btempo;
  N32=40000.0/Btempo;

  k = HOSTP_X(1);
  for (i=0; i<32; i++) 
  	{ Entab[i]=k&1; k>>=1; }
  k = HOSTP_X(2);
  for (i=32; i<NCHANS; i++) 
  	{ Entab[i]=k&1; k>>=1; }
  MSH0=Martpar;
  MSH=Martpar;
  Mpar2=Martpar;

  SoloIns = -1;
  for (i=0; i<NCHANS; i++) {
  	Busy[i]=0;
  	if ((SoloIns<0) && (IFLAGS(i)&I_FLG_SOLO)) SoloIns=i;
  }

  i=0;
  while (i<128) {
    k = HOSTP_I(1);
    for (j=7; j>=0; --j) {
      Mode[i++] = ((k&15)!=0);
      k >>= 4;
    }
    k = HOSTP_I(2);
    for (j=3; j>=0; --j) {
      Mode[i++] = ((k&15)!=0);
      k >>= 4;
    }
  }
  memcpy(Smode,Mode,128);

  if (HOSTP_I(3)==0) HOSTP_I(3)=300;

  k = HOSTP_X(3);
  Section = (k==0? 2:(k&7));
}


unsigned inrange(int n, unsigned ins, unsigned k)
{
  int lb = MINRANGE(ins);
  int hb = MAXRANGE(ins);

  if (n<lb) {
    n += (((lb-n)/12+1)*12);
  }
  else if (n>hb) {
    n -= (((n-hb)/12+1)*12);
  }
  if (k) { while (Mode[n]==0) n=(n+1)&127; }
  return(n);
}


/* ================================================================== */


static void ckreinit(void)
{
  static int cnt = 0;

  if ((++cnt) >= HOSTP_I(3)) {
    cnt = 0;
    Martpar=MSH0;
    MSH=MSH0;
    Mpar2=MSH0;
    srand(Rseed);
  }
}


unsigned vnote (NoteEvent *N,unsigned t)
{
  static int vtb[8] = { 342,342,343,342,343,343,342,342 };

  if (N->timbre==SoloIns) {
    if ((t&15)==0) {
      ocparam(SoloIns,vtb[((t>>4)&7)],t);
    }
    return note(N,t);
  }
  else {
    if (N->duration < 100) return(t);
    if ((N->timbre>31)&&((rand()%100)==0)) {
      int k = (rand()%225)+1;
      ocparam(N->timbre,(k<129? k:k+128),t);
    }
    ckreinit();
    return note(N,t);
  }
}


unsigned vrepnote (NoteEvent *N,unsigned iT,int dT,unsigned t)
{
  if (N->timbre==SoloIns)
    return repnote (N,iT,dT,t);
  else {
    if ((N->timbre>=32) || (N->duration < 100)) return(t);
    ckreinit();
    return repnote (N,iT,dT,t);
  }
}


/* ================================================================== */


unsigned noplay(NoteEvent *N,unsigned t)
{
 unsigned t1;

  t1=N->deltat*TimeD;
  if (t1>500000) t1=500000;
  t1 += t;
  Busy[N->timbre]=t1;
  return(t1);
}


unsigned trill (NoteEvent *N,unsigned t)
{
 int i,r,nt,ht,dv,iH,Mel;
 unsigned t1;

  if (t<Busy[N->timbre]) return t;
  if (N->pitch<40) return(noplay(N,t));

  Mel = 1;
  if (N->timbre!=SoloIns) {
    iH = N->intensity / 2; if (iH<(MINVEL+10)) return(t);
  }
  dv=(r=rand())&1;

  nt=(r%9)+12;
  ht=(r%2)+1;

  if (N->timbre==SoloIns) {
    N->duration = 32;
    N->deltat = 48+(t&7);
  }
  else {
    N->duration >>= (Mel? 3:2);
    if (N->duration>N32) N->duration=N32;
    N->deltat = N->duration+10;
  }

  t1=t;
  N->intensity=iH;
  for (i=0; i<nt; i++) {
    t1 = vnote(N,t1);
    N->pitch += ht;
    t1 = vnote(N,t1);
    N->pitch -= ht;
    if (dv) iH-=2; else iH++; if (iH<MINVEL) iH=MINVEL;
   }
  Busy[N->timbre]=t1;
  return(t+100);
}


unsigned mtrill (NoteEvent *N,unsigned t)
{
 int i,r,nt,ht,dv,rh,Mel;
 unsigned t1;

  if (t<Busy[N->timbre]) return t;
  if (N->pitch<36) return(noplay(N,t));

  Mel = 1;
  N->intensity /= 2;
  if (N->intensity<MINVEL) return(t);

  dv=(r=rand())&1;
  rh=Rseed;
  nt=(r&15)+6;
  ht=(r%3)+1;

  if (N->timbre==SoloIns) {
    N->duration = 30;
    N->deltat = 100;
  }
  else {
    N->duration /= (Mel? 6:5);
    if (N->duration>N32) N->duration=N32;
    N->deltat = N->duration+10;
  }

  t1=t;
  for (i=0; i<nt; i++) {
    if (rh&1) { t1 = vnote(N,t); N->pitch += onefrand(-7,7); }
    if (rh&2) { t1 = vnote(N,t); N->pitch += onefrand(-7,7); }
    if ((rh&3)==0) t1 += N->deltat*TimeD;
    rh = (rh>>2)|((rh&3)<<30);
    if (dv) N->intensity-=2; else N->intensity++;
    if (N->intensity<MINVEL) N->intensity=MINVEL;
   }
  Busy[N->timbre]=t1;
  return(t+100);
}


unsigned scale (NoteEvent *N,unsigned t)
{
 int i,k,r,nt,du,sgn;
 unsigned t1;

  if (t<Busy[N->timbre]) return t;
  sgn = (r=rand())&1;
  nt=4+(r%7);
  k=r&7;
  if (N->timbre==SoloIns) {
    N->duration = 30;
    N->deltat = 60+(t&15);
  }
  else {
    N->deltat = du = N->duration;
    N->duration = du/2;
  }

  t1=t;
  for (i=0; i<nt; i++) {
    if (sgn) {
      t1 = vnote(N,t1);
      N->pitch += ((rand()%3)+1);
      if (i&1) { while (Mode[N->pitch]==0) N->pitch++; }
      N->intensity--;
     }
    else {
      t1 = vnote(N,t1);
      N->pitch -= ((rand()%3)+1);
      if (i&1) { while (Mode[N->pitch]==0) N->pitch--; }
      N->intensity--;
     }
    N->pitch = inrange(N->pitch,N->timbre,0);
   }
  Busy[N->timbre]=t1;
  return(t+100);
}


static unsigned HscT;


unsigned pscale (NoteEvent *N,unsigned t)
{
 int i,r,nt,np;
 unsigned t1;

  if (t<Busy[N->timbre]) return t;
  HscT=t;
  nt=4+((r=rand())%11);
  if (N->timbre==SoloIns) {
    N->duration = 30;
    N->deltat = 64+(t&15);
  }
  else {
    N->deltat = N->duration+10;
  }
  np = (int)N->pitch-36;
  while (np<MINRANGE(N->timbre)) np+=12;

  t1=t;
  for (i=0; i<nt; i++) {
    np += onefrand(-4,7);
    while (Mode[np]==0) np++;
    if ((np>MAXRANGE(N->timbre))||(np<MINRANGE(N->timbre))) return(t1);
    N->pitch=np;
    t1 = vnote(N,t1);
    N->intensity--;
   }
  HscT=t1;
  Busy[N->timbre]=t1;
  return(t+100);
}


unsigned ggroup (NoteEvent *N,unsigned t)
{
 int i,j,r,nt,np;
 unsigned t0,t1;

  if (t<Busy[N->timbre]) return t;
  nt=((r=rand())%3)+1;
  N->duration = 30;
  N->deltat = 64+(t&15);

  np = (int)N->pitch-36;
  while (np<MINRANGE(N->timbre)) np+=12;

  t1=t;
  for (i=0; i<nt; i++) {
    N->pitch=np;
    t1 = vnote(N,t1);
    np = inrange(np+8+onefrand(-1,6),N->timbre,1);
    N->intensity+=2;
   }
  N->duration = 150;
  N->deltat = 200;
  t0 = vnote(N,t1);
  switch (j=(r&7)) {
    case 4:
    case 0: N->pitch = 55; vnote(N,t1); if (j==0) break;
    case 5:
    case 1: N->pitch = 62; vnote(N,t1); if (j==1) break;
    case 2: N->pitch = 69; vnote(N,t1); break;
  }
  Busy[N->timbre]=t0;
  return(t+100);
}


unsigned vgroup1 (NoteEvent *N,unsigned t)
{
 int i,j,r,nt,np;
 unsigned t0,t1;

  if (t<Busy[N->timbre]) return t;
  nt=((r=rand())%35)+1;
  N->duration = 25+(t&15);
  N->deltat = 46+(t&31);

  np = (int)N->pitch+1;
  while (np<MINRANGE(N->timbre)) np+=12;

  t1=t;
  for (i=0; i<nt; i++) {
    N->pitch=np;
    t0 = vnote(N,t1);
    switch (j=(rand()&7)) {
      case 4:
      case 0: N->pitch = 55; vnote(N,t1); if (j==0) break;
      case 5:
      case 1: N->pitch = 62; vnote(N,t1); if (j==1) break;
      case 2: N->pitch = 69; vnote(N,t1); break;
    }
    N->pitch=np+1;
    t1 = vnote(N,t0);
    np = inrange(np+onefrand(-6,6),N->timbre,1);
    N->intensity += (t&1);
   }
  Busy[N->timbre]=t1;
  return(t+100);
}


unsigned vgroup2 (NoteEvent *N,unsigned t)
{
 int i,r,nt,np;
 unsigned t1;

  if (t<Busy[N->timbre]) return t;
  nt=((r=rand())%13)+1;
  N->duration = 32+(t&7);
  N->deltat = 66+(t&15);

  np = (int)N->pitch+1;
  while (np<MINRANGE(N->timbre)) np+=12;

  t1=t;
  for (i=0; i<nt; i++) {
    N->pitch=np;
    if (t1&0x40) {
      note(N,t1);
      N->pitch=np+onefrand(-12,-6);
    }
    t1 = vnote(N,t1);
    N->pitch=np+onefrand(-3,3);
    if (t1&0x20) {
      note(N,t1);
      N->pitch=np+onefrand(-15,-3);
    }
    t1 = vnote(N,t1);
    np = inrange(np+onefrand(-6,6),N->timbre,1);
    N->intensity -= (t&1);
   }
  Busy[N->timbre]=t1;
  return(t+100);
}


unsigned vgroup3 (NoteEvent *N,unsigned t)
{
 int i,r,nt,np,dnp;
 unsigned t1;

  if (t<Busy[N->timbre]) return t;
  nt=((r=rand())%9)+1;
  N->duration = 32+(t&7);
  N->deltat = 50+(t&15);

  np = MINRANGE(N->timbre)+(r%6);

  t1=t;
  dnp=2;
  for (i=0; i<nt; i++) {
    N->pitch=np;
    t1 = vnote(N,t1);
    np = inrange(np+onefrand(0,dnp),N->timbre,1);
    dnp+=3;
    N->intensity -= (t&1);
   }
  Busy[N->timbre]=t1;
  return(t+100);
}


unsigned mscale (NoteEvent *N,unsigned t)
{
 int i,nt,np;
 unsigned t1;

  if (t<Busy[N->timbre]) return t;
  HscT=t;
  nt=4+(rand()%11);
  if (N->timbre==SoloIns) {
    N->duration = 50;
    N->deltat = 64+(t&31);
  }
  else {
    N->deltat = N->duration+10;
  }
  np = (int)N->pitch+36;
  while (np>MAXRANGE(N->timbre)) np-=12;

  t1=t;
  for (i=0; i<nt; i++) {
    np += onefrand(-7,4);
    while (Mode[np]==0) np--;
    if ((np>MAXRANGE(N->timbre))||(np<MINRANGE(N->timbre))) return(t1);
    N->pitch=np;
    t1 = vnote(N,t1);
    N->intensity--;
   }
  HscT=t1;
  Busy[N->timbre]=t1;
  return(t+100);
}


unsigned fgroup (NoteEvent *N,unsigned t)
{
 int i,nt,np,du,is,lastP;
 unsigned k,t1;

  if ((N->timbre>38)&&(N->timbre!=41)) return(t+5);
  if (t<Busy[N->timbre]) return t;
  nt=4+(rand()%7);
  np=N->pitch;

  if (N->timbre==SoloIns) {
    du = 80 + (t&63);
  }
  else {
    du = N->duration/5;
    if (du<N16) du=THIRTYSECOND; else if (du>QUARTER) du=QUARTER;
  }
  lastP=0;
  t1=t;

  is = (MINVEL+du)&127;
  if (is<MIDVEL) is=MIDVEL;
   else if (is>MAXVEL) is=MAXVEL;

  for (i=0; i<nt; i++) {
    N->pitch = np+onefrand(-15,15);
    k = (N->pitch/2)&3;
    switch(k) {
      case 0: while (Mode[N->pitch]==0) N->pitch++;
	      N->deltat = du;
	      N->duration = du+5;
	      break;
      case 1: while (Mode[N->pitch]==0) N->pitch--;
	      N->deltat = du*2;
	      N->duration = (du*2)+5;
	      t1=t;
	      break;
      case 2: while (Mode[N->pitch]==0) N->pitch++;
	      N->deltat = du/2;
	      N->duration = (du/2)+5;
	      break;
      case 3: while (Mode[N->pitch]==0) N->pitch--;
	      N->deltat = du;
	      N->duration = du+5;
	      break;
     }
    N->pitch = inrange(N->pitch,N->timbre,1);
    if (N->pitch!=lastP) t1 = vnote(N,t1);
    lastP=N->pitch;
    Busy[N->timbre]=t1;
   }
  return(t1);
}


unsigned block (NoteEvent *N,unsigned t)
{
 int i,k,np,is,mxd;
 unsigned t1,dt;
 int durs[NCHANS];

  np=N->pitch+12;
  is=(~N->intensity)&63;

  mxd=0;
  for (i=0; i<NCHANS; i++) {
    durs[i]=((rand()%15)+1)*THIRTYSECOND;
    if (durs[i]>mxd) mxd=durs[i];
   }
  if (Section==3) mxd /= 2;
  N->deltat=mxd+QUARTER;

  k=rand();
  dt = 0;
  for (i=0; i<32; i++) {
    N->timbre=i;
    N->duration=durs[i];
    if (k&1) {
      if (rand()&1) {
	N->pitch=inrange(np+onefrand(-21,-3),i,1);;
	while (Mode[N->pitch]==0) N->pitch++;
       }
      else {
	N->pitch=inrange(np+onefrand(3,21),i,1);;
	while (Mode[N->pitch]==0) N->pitch--;
       }
      N->intensity=is+onefrand(-10,2);
      t1=vnote(N,t+dt); t+=10;
     }
    k>>=1;
    if (Section==6) dt=durs[31-i]/2; else dt=durs[rand()&31];
   }
  k=rand() & 0x27f;
  for (i=32; i<NCHANS; i++) {
    N->timbre=i;
    N->duration=durs[i];
    if (k&1) {
      if (rand()&1) {
	N->pitch=inrange(np+onefrand(-21,-3),i,1);;
	while (Mode[N->pitch]==0) N->pitch++;
       }
      else {
	N->pitch=inrange(np+onefrand(3,21),i,1);;
	while (Mode[N->pitch]==0) N->pitch--;
       }
      N->intensity=is+onefrand(-10,2);
      t1=vnote(N,t+dt); t+=10;
     }
    k>>=1;
    if (Section==6) dt=durs[rand()&31+32]; else dt=durs[(NCHANS-1)-i]/2;
   }
  return(t1);
}


unsigned melody (NoteEvent *N,unsigned t)
{
 int i,j,nt,np,xt,is;
 Mhop *M = &Martpar;
 NoteEvent N1;
 unsigned t0,t1,du;

  if (t<Busy[N->timbre]) return t;
  N1=*N;
  xt=(rand()&3)+1;
  is=N1.intensity+((2-xt)*4);
  if (is<MIDVEL) is=MIDVEL;
   else if (is>MAXVEL) is=MAXVEL;
  nt=7+(rand()%23);
  np=N1.pitch;

  t0=t1=t+10;
  for (i=0; i<nt; i++) {
    martin(M);
    np += (M->iy<0? M->sg*12:(M->ix%7));
    N1.pitch = np = inrange(np,N1.timbre,(N1.timbre!=SoloIns));
    du=(t&1? THIRTYSECOND:((M->iy&3)+1)*xt*THIRTYSECOND);
    if (du>QUARTER) du=QUARTER;
    N1.duration=du;
    N1.deltat=du+10;
    N1.intensity=(is+(2-xt)*i);
    if ((N1.intensity>MINVEL)&&(N1.intensity<MAXVEL)) {
      t = vnote(&N1,t);
      if (N1.timbre==SoloIns) {
        int dq5 = rand()%5;
        switch (j=(rand()&15)) {
          case 4:
          case 0: N1.pitch = 55+dq5; vnote(&N1,t); if (j==0) break;
          case 11:
          case 3: N1.pitch = 62+dq5; vnote(&N1,t); if (j==3) break;
          case 7: N1.pitch = 69+dq5; vnote(&N1,t); break;
        }
      }
      if ((N1.timbre>=32)||(i<(nt/2))) t1=t;
    }
   }
  Busy[N->timbre]=t1;
  return(t0+1000);
}


/* ================================================================== */


void GenLoop(void)
{
 int Ntb,oNtb,i,j,k,iter,Tofs,longnote,jctr;
 int lastIk;
 unsigned t0;
 Mhop *M = &Martpar;
 NoteEvent N,N1;

  N.startvol=N.deltavol=0;
  N.startpress=N.deltapress=0;
  N.pitch=0;
  lastIk = -1;

  for (i=0; i<4; i++) martin(M);

  asm { ldtimer; ldc 6; shl; ldc 100000; sum; st t0; }
  t0=ntempo(TimeD,t0);
  iter=1;
  jctr=0;
  Tofs=33;

  for (;;) {

    do {
      martin(M);
      N.pitch = (int)(M->ix-M->iy+KEYCENT)&127;
      Ntb = (int)(Tofs+abs(M->ix)%10+M->sg)&CHANMASK;
      if ((Ntb>38)&&(Ntb!=41)) {
      	  if (SoloIns>=0) Ntb=SoloIns;
      	    else Ntb-=31;
      }
    }
    while (Entab[Ntb]==0);
    N.timbre = Ntb;

    N.pitch = inrange(N.pitch,Ntb,0);

    oevent(M->ix,M->iy,t0);

    longnote = (M->iy < 0);
    if (longnote) {
      N.duration = ((abs(M->iy)&7)+1)*QUARTER;
      N.intensity = MINVEL;
    }
    else {
      int ik;
      N.duration = ((M->iy&3)+1)*SIXTEENTH;
      ik = M->iy % 5;
      if (lastIk<0) N.intensity=lastIk=MIDVEL;
       else {
         N.intensity = lastIk + ik - 2;
         if (N.intensity<MINVEL) N.intensity=MINVEL;
           else if (N.intensity>MAXVEL) N.intensity=MAXVEL;
         lastIk = N.intensity;
       }
    }
    if ((signed)N.duration <= 0) N.duration = QUARTER;

    N.deltat = ((N.duration+(abs(M->ix)%500))/250)*250;
    if (N.deltat<N.duration) N.deltat=N.duration;

    N1=N;
    N1.intensity -= 20;
    if (N1.intensity<MINVEL) N1.intensity=MINVEL;
     else if (N1.intensity>MAXVEL) N1.intensity=MAXVEL;

    if ((Ntb==SoloIns)||(ISPIANO(Ntb))) {
      int isave=SoloIns;
      SoloIns=Ntb;
      Section=0;
      switch ((k=amrand()>>2)&15) {
	      case 10: t0 = trill(&N,t0); break;
	      case  7: t0 = scale(&N,t0); break;
	      case  0:
	      case  2: t0 = pscale(&N,t0); break;
	      case  3: t0 = mscale(&N,t0); break;
	      case  1:
	      case  4: t0 = fgroup(&N,t0); break;
	      case  6:
	      case 11: t0 = ggroup(&N,t0); break;
	      case  9:
	      case 12: t0 = vgroup1(&N,t0); break;
	      case 13: t0 = vgroup2(&N,t0); break;
	      case 14: t0 = vgroup3(&N,t0); break;
	      default: Section = 4+rand()%3; break;
      }
      SoloIns=isave;
    }
    switch (Section&7) {
      case 1:	    if ((rand()%3)==0) {
		      N1.pitch = N.pitch+onefrand(-4,-1);
		      N1.lock = 0;
		      vnote(&N1,t0);
		     }
		    N.lock = 0;
		    t0 = vnote(&N,t0);
		    break;

      case 2:	    if ((rand()%7)==1) {
		      N1.pitch = N.pitch+onefrand(-9,-4);
		      N1.lock = 0;
		      vnote(&N1,t0); t0+=10;
		     }
		    if ((rand()%3)==2) {
		      N1.pitch = N.pitch+onefrand(4,9);
		      N1.lock = 0;
		      vnote(&N1,t0); t0+=10;
		     }
		    N.lock = 0;
		    t0 = fgroup(&N,t0);
		    break;

      case 3:	    N.lock = 0;
      		    N.deltat <<= 4;
		    t0 = block(&N,t0);
		    if ((++jctr)>=8) { Section=rand()&7; jctr=0; }
		    break;

      case 4:	    N.lock = 0;
		    t0 = melody(&N,t0);
		    if ((++jctr)>=8) { Section=rand()&7; jctr=0; }
		    break;

      case 5:	    N.lock = 0;
      		    switch ((Ntb+N.pitch)%5) {
		      case 0: t0 = trill(&N,t0); break;
		      case 1: t0 = scale(&N,t0); break;
		      case 2: t0 = pscale(&N,t0); break;
		      case 3: t0 = mscale(&N,t0); break;
		      case 4: t0 = fgroup(&N,t0); break;
		    }
		    if ((++jctr)>=16) { Section=rand()&7; jctr=0; }
		    break;

      case 6:	    N.lock = 0;
      		    switch ((Ntb-N.pitch)&3) {
		      case 0: t0 = trill(&N,t0); break;
		      case 1: t0 = mtrill(&N,t0); break;
		      case 2: t0 = block(&N,t0); break;
		      case 3: t0 = melody(&N,t0); break;
		    }
		    if ((++jctr)>=24) { Section=rand()&7; jctr=0; }
		    break;

      case 7:	    if ((rand()%3)==0) {
		      N1.pitch = N.pitch+onefrand(-7,-3);
		      N1.lock = 0;
		      vnote(&N1,t0); t0+=10;
		     }
		    if ((rand()%3)==1) {
		      N1.pitch = N.pitch+onefrand(3,8);
		      N1.lock = 0;
		      vnote(&N1,t0); t0+=10;
		     }
		    N.lock = 0;
		    t0 = vnote(&N,t0);
      default:	    break;

     }

    oNtb = Ntb;
    t0 += 1000;
    iter++;

    if (((iter-Ntb)&0x7)==0) {
      char buf[128];
      j = rand()&7;
      if (j) {
        memcpy(buf+j,Mode,128-j);
        memcpy(buf,Mode+128-j,j);
	if (rand()&1)
          memcpy(Mode,buf,128);
        else {
          int ks=0;
          for (k=0; k<128; k++) { Mode[k] = (Mode[k]^buf[k])&Smode[k];
          	ks+=Mode[k]; }
          if (ks<3) memcpy(Mode,Smode,128);
        }
      }
      else {
        for (k=0; k<NCHANS; k++) t0=allnoteoff(k,t0+5);
      }
    }
    if (HostCmd&0x80) {
/* ================================================================== */
/*		host command handling				      */
/* ================================================================== */
      switch(HostCmd&0x7f) {
	case 27:  return;
	case '.': Martpar=MSH;
		  srand(Rseed);
		  break;
	case ',': MSH=Martpar;
		  break;
	case '?': k = RTAB%12; if (k<2) k=12;
		  for (i=13; i<96; i+=k)
		    Mode[i] ^= 1;
		  Section++;
		  break;
	case '/': for (i=13; i<96; i++)
		    Mode[i] ^= 1;
		  break;
	case 'R': for (k=0; k<100; k++) martin(M);
		  break;
	case 'G': TimeD /= 4; if (TimeD<LIMD_L) TimeD=LIMD_L;
		  t0=ntempo(TimeD,t0);
		  Btempo /= 4;
		  N16=80000.0/Btempo;
		  N32=40000.0/Btempo;
		  break;
	case 'F': TimeD /= 1.5; if (TimeD<LIMD_L) TimeD=LIMD_L;
		  t0=ntempo(TimeD,t0);
		  break;
	case 'Q': for (k=0; k<NCHANS; k++) t0=allnoteoff(k,t0+5);
		  break;
	case 'D': TimeD *= 4; if (TimeD>LIMD_H) TimeD=LIMD_H;
		  t0=ntempo(TimeD,t0);
		  Btempo *= 4;
		  N16=80000.0/Btempo;
		  N32=40000.0/Btempo;
		  break;
	case 'S': TimeD *= 1.5; if (TimeD>LIMD_H) TimeD=LIMD_H;
		  t0=ntempo(TimeD,t0);
		  break;
	case 'Z': Martpar=MSH0;
		  MSH=MSH0;
		  Mpar2=MSH0;
		  srand(Rseed);
	case 'A': TimeD = Btempo;
		  t0=ntempo(TimeD,t0);
		  break;
	case '0':
	case '1':
	case '2':
	case '3':
	case '4': Section=HostCmd&7; break;
	case '5':
	case '6':
	case '7': Section=HostCmd&7;
		  for (k=0; k<NCHANS; k++) t0=allnoteoff(k,t0+5);
	default:  M->x = HostCmd/128.0;
		  M->y = HostCmd/127.0;
		  srand(Rseed);
		  break;
/* ================================================================== */
       }
      HostCmd=(iter&0x7f);
     }
    asm { ld t0; ldc 6; shr; adc -100; tin; }

   }
}


/* ================================================================== */

