
/* ================================================================== */
/*		NOTE GENERATION UNIT: Martin's Hopalong               */
/*			version 003                                   */
/* ================================================================== */

#include "../tsrc/mplay.h"
#include "../proteus.h"
#include "../moc1.h"
#include "martin.h"

#define KEYCENT		60
#define MINKEY		28
#define MAXKEY		90
#define MINVEL		16
#define MIDVEL		76
#define MAXVEL		110

#define MINDELTA	5
#define MAXDELTA	2000

#define LIMD_L		500
#define LIMD_H		5000

#define RTAB		((rand()&63)+24)

Mhop  Martpar;

int  Tbtab[NCHANS];
char Mode[128];

#define TRUENCHANS	NCHANS
#define TRUECHANMASK	CHANMASK

/* ================================================================== */


void martin(Mhop *M)
{
 float t,u;
 int f;

  t=M->b*M->x-M->c; if (t<0) t= -t;
  if (M->x<0) u=M->y+sqrt(t);
   else if (M->x>0) u=M->y-sqrt(t);
    else u=M->y;
  M->y = M->a-M->x;
  M->x = u;
  f=0;

  if (M->x<M->lx) { M->lx=M->x; f=1; }
   else if (M->x>M->hx) { M->hx=M->x; f=1; }
  if (f)
    M->sc = 60/(M->hx-M->lx);

  M->ix = M->x*M->sc;
  M->iy = M->y*M->sc;

  M->sq = sqrt(M->ix*M->ix+M->iy*M->iy);
}


unsigned inrange(int n, unsigned lb, unsigned hb)
{
  if (n<lb) {
    n += (((lb-n)/12+1)*12);
  }
  else if (n>hb) {
    n -= (((n-hb)/12+1)*12);
  }
  while (Mode[n]==0) n=(n+1)&127;
  return(n);
}


/* ================================================================== */


void InitGen(void)
{
 int k,r,t;
 float f;

  Martpar.a = HOSTP_F(0);
  Martpar.b = HOSTP_F(1);
  Martpar.c = HOSTP_F(2);
  Martpar.x = 0;
  Martpar.y = 0;
  Martpar.lx = 0;
  Martpar.hx = 0;
  Martpar.sc = HOSTP_I(0);
  
  f = Martpar.a*Martpar.b*Martpar.c;
  f1init(f);
  _memcpy(&k,&f,4);
  Rseed=k;
  srand(k);

  if (f==0) f=150;
   else if (f<0) f = -f;
  while (f<150) f*=2;
  while (f>3000) f/=3;
  Btempo=f;

  if (HOSTP_I(1)) {
    for (k=16; k<111; k++) Mode[k] = (((k%5)!=0)&&((k%7)==0));
  }
  else
    for (k=0; k<128; k++) Mode[k] = Params.Mtempl[k];

  for (k=0; k<TRUENCHANS; k++) Tbtab[k]=k;
  for (k=0; k<TRUENCHANS; k++) {
    r=rand()&TRUECHANMASK;
    t=Tbtab[r]; Tbtab[r]=Tbtab[1]; Tbtab[1]=t;
   }
}


/* ================================================================== */


unsigned zcount=0;


void GenLoop(void)
{
 int Ntb,i,k,mxv,iter,Oldt,Tofs;
 unsigned t0,t1,maxt;		/* microsec */
 Mhop *M = &Martpar;
 NoteEvent N;

  mxv=0; N.pitch=0; N.a=0; N.b=0; N.c=0;
  N.startvol=N.deltavol=0;
  N.startpress=N.deltapress=0;
  N.lock=0;

  k=HOSTP_I(2); if (k<=4) k=4;
  for (i=0; i<k; i++) martin(M);

  asm { ldtimer; ldc 6; shl; ldc 10000; sum; st t0; }
  t0=ntempo(TimeD,t0);
  maxt=t0;
  iter=0; Oldt=0;
  Tofs = rand()&TRUECHANMASK;

  for (;;) {

    martin(M);

    N.pitch = M->ix+KEYCENT;

    Ntb = (Tofs+(N.pitch&3)+(iter>>7))&TRUECHANMASK;
    if (Ntb>39) Ntb=32+(Ntb&7);

    if ((rand()%3)==0) {
      Ntb=Oldt; N.deltat=5;
     }
    else {
      N.deltat = (Ntb+2)*SIXTEENTH+(M->iy*64);
      if (N.deltat>MAXDELTA) N.deltat=MAXDELTA;
       else if (N.deltat<MINDELTA) N.deltat=MINDELTA;
     }
    Oldt=Ntb;

    k = ((int)(M->sq/4)%8)+(15-(Ntb&15));
    N.timbre=Ntb=Tbtab[Ntb];
    N.duration = (k+1)*EIGHTH;

    N.intensity = ((25-k)*2)+MINVEL;
    N.intensity += abs(N.pitch-KEYCENT)/2;
    if (N.intensity<MINVEL) N.intensity=MINVEL;
    if (N.intensity>MAXVEL) N.intensity=MAXVEL;
    if (N.intensity>mxv) mxv=N.intensity;

    N.pitch = inrange(N.pitch,MINRANGE(Ntb),MAXRANGE(Ntb));

    if (zcount==0) {
      t1 = t0+(N.duration*TimeD);
      if (t1>maxt) maxt=t1;		/* with duration */

      if (rand()&3)
	t0 = note(&N,t0);

      if (t0>maxt) maxt=t0;		/* with deltat */
     }

    iter++;

    if ((iter%11)==0) {
      k = RTAB%12;
      for (i=12; i<128; i+=12)
	Mode[i+k] ^= 1;
     }
    else if (HostCmd) {
      switch(HostCmd) {
        case 27:  return;
	case '>': zcount=100;
		  break;
	case 'R': for (k=0; k<100; k++) martin(M);
		  break;
	case 'F': TimeD /= 1.5; if (TimeD<LIMD_L) TimeD=LIMD_L;
		  t0=ntempo(TimeD,t0);
		  break;
	case 'D': TimeD *= 4; if (TimeD>LIMD_H) TimeD=LIMD_H;
		  t0=ntempo(TimeD,t0);
		  break;
	case 'G': TimeD /= 4; if (TimeD<LIMD_L) TimeD=LIMD_L;
		  t0=ntempo(TimeD,t0);
		  break;
	case 'S': TimeD *= 1.5; if (TimeD>LIMD_H) TimeD=LIMD_H;
		  t0=ntempo(TimeD,t0);
		  break;
	case 'Q': for (k=0; k<32; k++) t0=allnoteoff(k,t0+5);
		  break;
	case 'Z': M->x=0; M->y=0;
	case 'A': TimeD = Btempo;
		  t0=ntempo(TimeD,t0);
		  break;
	default:  M->x = HostCmd/128.0;
		  M->y = HostCmd/127.0;
		  Tofs = HostCmd&TRUECHANMASK;
		  srand(Rseed);
		  break;
       }
      HostCmd=0;
     }
    if (zcount) zcount--;
     else
    asm { ld t0; ldc 6; shr; adc -100; tin; }

   }
}


/* ================================================================== */

