
/* ================================================================== */
/*		NOTE GENERATION UNIT: Martin's Hopalong               */
/*			version 005a                                  */
/* ================================================================== */

#include "../tsrc/mplay.h"
#include "../proteus.h"
#include "../moc1.h"
#include "martin.h"

#define KEYCENT		62
#define MINKEY		28
#define MAXKEY		90
#define MINVEL		36
#define MIDVEL		60
#define MAXVEL		100
#define MINDELTA	5
#define MAXDELTA	2000

#define LIMD_L		150
#define LIMD_H		3000

#define RTAB		((rand()&63)+24)

Mhop  Martpar,MSH,MSH0;
int   N16, N32, Etb;

int  Entab[NCHANS], Busy[NCHANS];

char Mode[128];

int Section;

/* ================================================================== */

unsigned trill  (NoteEvent *N,unsigned t);
unsigned mtrill (NoteEvent *N,unsigned t);
unsigned scale  (NoteEvent *N,unsigned t);
unsigned pscale (NoteEvent *N,unsigned t);
unsigned mscale (NoteEvent *N,unsigned t);
unsigned fgroup (NoteEvent *N,unsigned t);
unsigned block  (NoteEvent *N,unsigned t);
unsigned melody (NoteEvent *N,unsigned t);
unsigned noplay (NoteEvent *N,unsigned t);

/* ================================================================== */
/*		FORMULA: Martin's Hopalong			      */
/* ================================================================== */

void martin(Mhop *M)
{
 float t,u;
 int f;

/* iterazione algoritmo */
  t=M->b*M->x-M->c; if (t<0) t= -t;
  if (M->x<0) u=M->y+sqrt(t);
   else if (M->x>0) u=M->y-sqrt(t);
    else u=M->y;
  M->y = M->a-M->x;
  M->x = u;

/* calcolo scala e correzione */
  f=0;
  if (M->x<M->lx) { M->lx=M->x; f=1; }
   else if (M->x>M->hx) { M->hx=M->x; f=1; }
  if (f)
    M->sc = 32/(M->hx-M->lx);

  M->ix = M->x*M->sc;
  M->sg = (M->ix==0? 0:(M->ix<0? -1:1));
  M->iy = M->y*M->sc;
  M->sg += (M->iy==0? 0:(M->iy<0? -1:1));

/* extra parameters */
  M->sq = sqrt(M->x*M->x+M->y*M->y);
  M->isq = sqrt(M->ix*M->ix+M->iy*M->iy);
}


/* ================================================================== */
/*		GENERATOR loop					      */
/*								      */
/*	Host parameter use:					      */
/*	F0 F1 F2		a,b,c				      */
/*	I0			scale factor			      */
/*	I1			mode rotation			      */
/*								      */
/* ================================================================== */

void InitGen(void)
{
 int i,j,k;
 float f;

  Martpar.a = -(HOSTP_F(0));
  Martpar.b = -(HOSTP_F(1));
  Martpar.c = -(HOSTP_F(2));
  Martpar.x = 0;
  Martpar.y = 0;
  Martpar.lx = 0;
  Martpar.hx = 0;
  Martpar.sc = HOSTP_I(0); if (Martpar.sc<=0) Martpar.sc=10;

  f = Martpar.a*Martpar.b*Martpar.c;
  f1init(f);
  _memcpy(&k,&f,4);
  Rseed=k;
  srand(k);

/* Btempo unit = 500-6000 us */
  if (f==0) f=2000;
   else if (f<0) f = -f;
  while (f<1000) f*=3;
  while (f>6000) f/=2;
  Btempo=f;
  N16=150000.0/Btempo;
  N32=75000.0/Btempo;

  k = HOSTP_X(1);
  for (i=0; i<32; i++) 
  	{ Entab[i]=k&1; k>>=1; }
  k = HOSTP_X(2);
  for (i=32; i<NCHANS; i++) 
  	{ Entab[i]=k&1; k>>=1; }
  MSH0=Martpar;
  MSH=Martpar;

  for (i=0; i<NCHANS; i++) Busy[i]=0;

  for (i=0; i<16; i++) {
    k = HOSTP_I(1);
    for (j=7; j>=0; --j) {
      Mode[i*8+j] = ((k&15)!=0);
      k >>= 4;
    }
  }
  Mode[62] = 1;

  Section = (rand()&1? 2:5);
}


unsigned inrange(int n, unsigned lb, unsigned hb)
{
  if (n<lb) {
    n += (((lb-n)/12+1)*12);
  }
  else if (n>hb) {
    n -= (((n-hb)/12+1)*12);
  }
  while (Mode[n]==0) n=(n+1)&127;
  return(n);
}


/* ================================================================== */


unsigned noplay(NoteEvent *N,unsigned t)
{
 unsigned t1;

  t1=N->deltat*TimeD;
  if (t1>500000) t1=500000;
  t1 += t;
  Busy[Etb]=t1;
  Etb++; Etb &= CHANMASK;
  return(t1);
}


unsigned trill (NoteEvent *N,unsigned t)
{
 int i,nt,ht,dv,iH,iL,Mel;
 unsigned t1;

  if (N->pitch<40) return(noplay(N,t));

  Mel = 1;
  iH = N->intensity / 2; if (iH<MINVEL) iH=MINVEL;
  iL = N->intensity / 3; if (iL<MINVEL) iL=MINVEL;
  dv=rand()&1;

  nt=(rand()%9)+12;
  ht=rand()%2+1;
  N->duration >>= (Mel? 3:2);
  if (N->duration>N32) N->duration=N32;
  N->deltat = N->duration+1;

  t1=t+1;
  for (i=0; i<nt; i++) {
    N->intensity=iH;
    t = note(N,t);
    N->pitch += ht;
    N->intensity=iL;
    t = note(N,t);
    N->pitch -= ht;
    if (dv) iH-=2; else iH++; if (iH<MINVEL) iH=MINVEL;
   }
  return(t1);
}


unsigned mtrill (NoteEvent *N,unsigned t)
{
 int i,nt,ht,dv,rh,Mel;
 unsigned t1;

  if (N->pitch<36) return(noplay(N,t));

  Mel = 1;
  N->intensity /= 2;
  if (N->intensity<MINVEL) N->intensity=MINVEL;

  dv=rand()&1;
  rh=Rseed;
  nt=(rand()&31)+6;
  ht=rand()%3+1;

  N->duration /= (Mel? 2:3);
  if (N->duration>N16) N->duration=N16;
  N->deltat = N->duration+1;

  t1=t+1;
  for (i=0; i<nt; i++) {
    if (rh&1) t = note(N,t); else t+=N->deltat*TimeD;
    if (rh&2) t = note(N,t); else t+=N->deltat*TimeD;
    rh = (rh>>2)|((rh&3)<<30);
    if (dv) N->intensity-=2; else N->intensity++;
    if (N->intensity<MINVEL) N->intensity=MINVEL;
   }
  return(t1);
}


unsigned scale (NoteEvent *N,unsigned t)
{
 int i,nt,du,sgn;
 unsigned t1;

  sgn = rand()&1;
  nt=4+(rand()%7);
  N->deltat = du;
  N->duration = du/2;

  t1=t+1;
  for (i=0; i<nt; i++) {
    if (sgn) {
      t = note(N,t);
      N->pitch = (N->pitch+((rand()%3)+1))&127;
      if (i&1) { while (Mode[N->pitch]==0) N->pitch=(N->pitch+1)&127; }
      N->intensity--;
     }
    else {
      t = note(N,t);
      N->pitch = (N->pitch-((rand()%3)+1))&127;
      if (i&1) { while (Mode[N->pitch]==0) N->pitch=(N->pitch-1)&127; }
      N->intensity--;
     }
    while (N->pitch<MINRANGE(Etb)) N->pitch+=12;
    while (N->pitch>MAXRANGE(Etb)) N->pitch-=12;
   }
  return(t1/2+t/2);
}


static unsigned HscT;


unsigned pscale (NoteEvent *N,unsigned t)
{
 int i,nt,np;
 unsigned t1;

  HscT=t;
  nt=4+(rand()%11);
  N->deltat = N->duration;
  N->duration -= 10;
  np = ((int)N->pitch-36)&127;
  while (np<MINRANGE(Etb)) np+=12;

  t1=t+1;
  for (i=0; i<nt; i++) {
    np = (np+(int)onefrand(-4,7))&127;
    while (Mode[np]==0) np=(np+1)&127;
    if ((np>MAXRANGE(Etb))||(np<MINRANGE(Etb))) return(t1);
    N->pitch=np;
    t = note(N,t);
    N->intensity--;
   }
  HscT=t;
  return(t1/3+(t/3)*2);
}


unsigned mscale (NoteEvent *N,unsigned t)
{
 int i,nt,np;
 unsigned t1;

  HscT=t;
  nt=4+(rand()%11);
  N->deltat = N->duration;
  N->duration -= 10;
  np = ((int)N->pitch+36)&127;
  while (np>MAXRANGE(Etb)) np-=12;

  t1=t+1;
  for (i=0; i<nt; i++) {
    np = (np+(int)onefrand(-7,4))&127;
    while (Mode[np]==0) np=(np-1)&127;
    if ((np>MAXRANGE(Etb))||(np<MINRANGE(Etb))) return(t1);
    N->pitch=np;
    t = note(N,t);
    N->intensity--;
   }
  HscT=t;
  return((t1/3)*2+t/3);
}


unsigned fgroup (NoteEvent *N,unsigned t)
{
 int i,nt,np,du,is,lastP;
 unsigned k,t1;

  nt=4+(rand()%7);
  np=N->pitch;

  du = N->duration/5;
  if (du<N16) du=THIRTYSECOND; else if (du>QUARTER) du=QUARTER;
  lastP=0;
  t1=t+10;

  is = (MINVEL+du)&127;
  if (is<MIDVEL) is=MIDVEL;
   else if (is>MAXVEL) is=MAXVEL;

  for (i=0; i<nt; i++) {
    N->pitch = (np+(int)onefrand(-15,15))&127;
    k = (N->pitch/2)&3;
    switch(k) {
      case 0: while (Mode[N->pitch]==0) N->pitch=(N->pitch+1)&127;
	      N->deltat = du;
	      N->duration = du+5;
	      break;
      case 1: while (Mode[N->pitch]==0) N->pitch=(N->pitch-1)&127;
	      N->deltat = du*2;
	      N->duration = (du*2)+5;
	      t1=t;
	      break;
      case 2: while (Mode[N->pitch]==0) N->pitch=(N->pitch+1)&127;
	      N->deltat = du/2;
	      N->duration = (du/2)+5;
	      break;
      case 3: while (Mode[N->pitch]==0) N->pitch=(N->pitch-1)&127;
	      N->deltat = du;
	      N->duration = du+5;
	      break;
     }
    while (N->pitch<MINRANGE(Etb)) N->pitch+=12;
    while (N->pitch>MAXRANGE(Etb)) N->pitch-=12;
    if (N->pitch!=lastP) t = note(N,t);
    lastP=N->pitch;
    Busy[Etb]=t;
   }
  return(t1);
}


unsigned block (NoteEvent *N,unsigned t)
{
 int i,k,np,is,mxd;
 unsigned t1;
 int durs[NCHANS];

  np=(N->pitch+12)&127;
  is=(~N->intensity)&63;

  mxd=0;
  for (i=0; i<NCHANS; i++) {
    durs[i]=((rand()%11)+1)*SIXTEENTH;
    if (durs[i]>mxd) mxd=durs[i];
   }
  N->deltat=mxd+THIRTYSECOND;

  k=rand();
  for (i=0; i<32; i++) {
    N->timbre=i;
    N->duration=durs[i];
    if (k&1) {
      if (rand()&1) {
	N->pitch=(np+(int)onefrand(-11,-3))&127;
	while (Mode[N->pitch]==0) N->pitch=(N->pitch+1)&127;
       }
      else {
	N->pitch=(np+(int)onefrand(3,11))&127;
	while (Mode[N->pitch]==0) N->pitch=(N->pitch-1)&127;
       }
      while (N->pitch<MINRANGE(i)) N->pitch+=12;
      while (N->pitch>MAXRANGE(i)) N->pitch-=12;
      N->intensity=is+onefrand(-10,2);
      t1=note(N,t); t+=10;
     }
    k>>=1;
   }
  k=rand() & 0xffff007f;
  for (i=32; i<NCHANS; i++) {
    N->timbre=i;
    N->duration=durs[i];
    if (k&1) {
      if (rand()&1) {
	N->pitch=(np+(int)onefrand(-11,-3))&127;
	while (Mode[N->pitch]==0) N->pitch=(N->pitch+1)&127;
       }
      else {
	N->pitch=(np+(int)onefrand(3,11))&127;
	while (Mode[N->pitch]==0) N->pitch=(N->pitch-1)&127;
       }
      while (N->pitch<MINRANGE(i)) N->pitch+=12;
      while (N->pitch>MAXRANGE(i)) N->pitch-=12;
      N->intensity=is+onefrand(-10,2);
      t1=note(N,t); t+=10;
     }
    k>>=1;
   }
  return(t1);
}


unsigned melody (NoteEvent *N,unsigned t)
{
 int i,nt,np,du,xt,is;
 Mhop *M = &Martpar;
 NoteEvent N1;
 unsigned t1;

  N1=*N;
  xt=(rand()&3)+1;
  is=N1.intensity+((2-xt)*4);
  if (is<MIDVEL) is=MIDVEL;
   else if (is>MAXVEL) is=MAXVEL;
  nt=7+(rand()%11);
  np=N1.pitch;
  t1=t+10;

  for (i=0; i<nt; i++) {
    martin(M);
    N1.pitch = (MINRANGE(Etb)+(M->ix&63))&127;
    while (N1.pitch>MAXRANGE(Etb)) N1.pitch-=12;
    du=(t&1? THIRTYSECOND/2:((M->iy&3)+1)*xt*THIRTYSECOND);
    if (du>WHOLE) du=WHOLE;
    N1.duration=du;
    N1.deltat=du;
    N1.intensity=(is+(2-xt)*i);
    if ((N1.intensity>MINVEL)&&(N1.intensity<MAXVEL)) {
      t = note(&N1,t);
      if (i<(nt/2)) t1=t;
    }
   }
  return(t1);
}


/* ================================================================== */


void GenLoop(void)
{
 int Ntb,i,j,k,iter,Tofs,Nofs,longnote,jctr;
 int rIt, rDt;
 unsigned t0;
 Mhop *M = &Martpar;
 NoteEvent N,N1;

  N.startvol=N.deltavol=0;
  N.startpress=N.deltapress=0;
  N.pitch=0;

  for (i=0; i<4; i++) martin(M);
  Nofs=0;
  Tofs = rand()&CHANMASK;

  asm { ldtimer; ldc 6; shl; ldc 100000; sum; st t0; }
  t0=ntempo(TimeD,t0);
  iter=1;
  jctr=0;

  for (;;) {

    martin(M);

    N.pitch = (int)(M->sq+Nofs-(N.intensity/4)+KEYCENT)&127;

    Etb = Ntb = (Tofs+((M->ix-M->sg)&0x33))&CHANMASK;
    if ((Etb>39)&&(Etb<48)) Etb -= 24;

    oevent(M->ix,M->iy,t0);

    N.deltat = (Ntb%3)*EIGHTH;
    
    longnote = (M->iy < 0);
    if (longnote)
      N.duration = ((M->iy&7)+2)*HALF;
    else
      N.duration = (((M->iy%7)&7)+2)*SIXTEENTH;
    if ((signed)N.duration <= 0) N.duration = QUARTER;

    N.timbre=Ntb=Etb;

    if (!Entab[Ntb]) {
      t0 = nothing(&N,t0); goto lnext;
    }

    if (longnote) {
      N.intensity = MINVEL;
    }
    else {
      N.intensity = (M->isq%(MAXVEL-MIDVEL))+MIDVEL;
      if (N.intensity<MINVEL) N.intensity=MINVEL;
       else if (N.intensity>MAXVEL) N.intensity=MAXVEL;
    }

    N.pitch = inrange(N.pitch,MINRANGE(Ntb),MAXRANGE(Ntb));

    N1=N;
    N.intensity -= 20;
    if (N.intensity<MINVEL) N.intensity=MINVEL;
     else if (N.intensity>MAXVEL) N.intensity=MAXVEL;

    switch (Section&7) {
      case 1:	    if ((rand()%3)==0) {
		      N1.pitch = N.pitch+onefrand(-4,-1);
		      N1.lock = 0;
		      note(&N1,t0);
		     }
		    N.lock = 0;
		    rIt = 375000+(N1.duration<<6)-((88-N1.pitch)<<8);	/* usec */
		    rDt = -((rand()&15)<<9);
		    t0 = repnote(&N,rIt,rDt,t0);
		    if ((++jctr)>=12) { Section=rand()&7; jctr=0; }
		    break;

      case 2:	    if ((rand()%7)==1) {
		      N1.pitch = N.pitch+onefrand(-9,-4);
		      N1.lock = 0;
		      rIt = 80000+(N1.duration<<5)-((88-N1.pitch)<<8);
		      rDt = (rand()&31)<<9;
		      repnote(&N1,rIt,rDt,t0); t0+=10;
		     }
		    if ((rand()%3)==2) {
		      N1.pitch = N.pitch+onefrand(4,9);
		      N1.lock = 0;
		      rIt = 64000+(N1.duration<<7)-((88-N1.pitch)<<8);
		      rDt = -((rand()&15)<<9);
		      repnote(&N,rIt,rDt,t0); t0+=10;
		     }
		    N.lock = 0;
		    t0 = fgroup(&N1,t0);
		    break;

      case 3:	    N.lock = 0;
      		    N.deltat <<= 4;
		    t0 = block(&N,t0);
		    if ((++jctr)>=4) { Section=rand()&7; jctr=0; }
		    break;

      case 4:	    N.lock = 0;
		    t0 = melody(&N,t0);
		    if ((++jctr)>=8) { Section=rand()&7; jctr=0; }
		    break;

      case 5:	    N.lock = 0;
      		    switch ((Etb+N.pitch)%5) {
		      case 0: t0 = trill(&N,t0); break;
		      case 1: t0 = scale(&N,t0); break;
		      case 2: t0 = pscale(&N,t0); break;
		      case 3: t0 = mscale(&N,t0); break;
		      case 4: t0 = fgroup(&N,t0); break;
		    }
		    if ((++jctr)>=16) { Section=rand()&7; jctr=0; }
		    break;

      case 6:	    N.lock = 0;
      		    switch ((Etb+t0)%4) {
		      case 0: t0 = trill(&N,t0); break;
		      case 1: t0 = mtrill(&N,t0); break;
		      case 2: t0 = scale(&N,t0) + QUARTER; break;
		      case 3: t0 = melody(&N,t0); break;
		    }
		    if ((++jctr)>=16) { Section=rand()&7; jctr=0; }
		    break;

      default:	    if ((rand()%3)==0) {
		      N1.pitch = N.pitch+onefrand(-7,-3);
		      N1.lock = 0;
		      rIt = 128000+(N1.duration<<5)-((88-N1.pitch)<<8);
		      rDt = (rand()&31)<<9;
		      repnote(&N1,rIt,rDt,t0); t0+=10;
		     }
		    if ((rand()%3)==1) {
		      N1.pitch = N.pitch+onefrand(3,8);
		      N1.lock = 0;
		      rIt = 90000+(N1.duration<<6)-((88-N1.pitch)<<8);
		      rDt = -((rand()&31))<<9;
		      repnote(&N1,rIt,rDt,t0); t0+=10;
		     }
		    N.lock = 0;
		    t0 = fullnote(&N,t0);
		    break;

     }

lnext:
    iter++;

    if ((iter&0x1f)==(Ntb&31)) {
      for (k=0; k<NCHANS; k++) t0=allnoteoff(k,t0+5);
    }
    if ((iter&0xfd)==0) {
      k = rand()&127;
      j = k&1;
      if (j) {
	while (Mode[k]==0) k=(k+1)&127;
	Mode[(k-7)&127]=1;
      }
      else {
	while (Mode[k]==0) k=(k-1)&127;
	Mode[(k+7)&127]=1;
      }
     }
    else if (HostCmd) {
/* ================================================================== */
/*		host command handling				      */
/* ================================================================== */
      switch(HostCmd) {
	case 27:  return;
	case '.': Martpar=MSH;
		  srand(Rseed);
		  break;
	case ',': MSH=Martpar;
		  break;
	case '+': Nofs++; break;
	case '-': Nofs--; break;
	case '?': k = RTAB%12; if (k<2) k=12;
		  for (i=13; i<96; i+=k)
		    Mode[i] ^= 1;
		  Section++;
		  break;
	case '/': for (i=13; i<96; i++)
		    Mode[i] ^= 1;
		  break;
	case 'R': for (k=0; k<100; k++) martin(M);
		  break;
	case 'G': TimeD /= 4; if (TimeD<LIMD_L) TimeD=LIMD_L;
		  t0=ntempo(TimeD,t0);
		  break;
	case 'F': TimeD /= 1.5; if (TimeD<LIMD_L) TimeD=LIMD_L;
		  t0=ntempo(TimeD,t0);
		  break;
	case 'Q': for (k=0; k<NCHANS; k++) t0=allnoteoff(k,t0+5);
		  break;
	case 'D': TimeD *= 4; if (TimeD>LIMD_H) TimeD=LIMD_H;
		  t0=ntempo(TimeD,t0);
		  break;
	case 'S': TimeD *= 1.5; if (TimeD>LIMD_H) TimeD=LIMD_H;
		  t0=ntempo(TimeD,t0);
		  break;
	case 'Z': Martpar=MSH0;
		  srand(Rseed);
	case 'A': TimeD = Btempo;
		  t0=ntempo(TimeD,t0);
		  break;
	case '0':
	case '1':
	case '2':
	case '3':
	case '4': Section=HostCmd&7; break;
	case '5':
	case '6':
	case '7': Section=HostCmd&7;
		  for (k=0; k<NCHANS; k++) t0=allnoteoff(k,t0+5);
	default:  M->x = HostCmd/128.0;
		  M->y = HostCmd/127.0;
		  Tofs = HostCmd&CHANMASK;
		  srand(Rseed);
		  break;
/* ================================================================== */
       }
      HostCmd=0;
     }
    asm { ld t0; ldc 6; shr; adc -100; tin; }

   }
}


/* ================================================================== */

