
/* ==================================================================== */
/*	MSERV - general PC MIDI server - Linux version 10/94		*/
/* ==================================================================== */

/* Included files */

#include <unistd.h>
#include <fcntl.h>
#include <sys/stat.h>
#include <stdio.h>
#include <errno.h>
#include <ctype.h>
#include <string.h>
#include <math.h>
#include <signal.h>
#include <termios.h>
#include <linux/link.h>
#include <asm/io.h>

#include "../proteus.h"
#include "../canvas.h"
#include "../fm_def.h"

#ifdef DTC3
#include <linux/dtc3.h>
int	fM, fT;
int	fA[4];
#else
#include "mpu401.h"
#endif

#undef  LINKIRQ
#undef  TIMRIRQ
#define MIDIIRQ

/* ==================================================================== */
/*			default parameters				*/
/* ==================================================================== */

char fname[256] = "../tbin/martin5.btl";
char pname[256] = "";
char insname[256] = "";
char logname[256] = "";
char midname[256] = "";
char dmpname[256] = "/tmp/tserv.000";

_paramblk Params =

{
/* 4 float parameters */
	{ 1.0, 2.0, 4.0, 0.0 },
/* 4 integer parameters */
	{ 10, 0, 0, 0 },
/* control parameters */
	{ 0, 0, 0, 0, 0, 0, 0, 0 },
	{
	 { "Harp#1",	    0, 0,P2u_Harp,          100,0,24,96 },
	 { "Viola",	    0, 1,P2u_SoloViola,     100,0,48,72 },
	 { "StrPizz",	    0, 2,P2p_Pizzicato2,     90,0,24,96 },
	 { "Pizz Bass",	    0, 3,P2u_PizzBasses,    100,0,24,48 },
	 { "Flute",	    0, 4,P2u_Flutenv,       100,0,60,96 },
	 { "Oboe",	    0, 5,P2u_AltOboe,       100,0,52,80 },
	 { "Clarinet",	    0, 6,P2p_BassClarinet,   90,0,24,72 },
	 { "Brass",	    0, 7,P2u_ChamberBrass,   90,0,36,78 },
	 { "Cello",	    0, 8,P2u_SoloCello,     100,0,36,66 },
	 { "Celesta",	    0, 9,P2u_Celesta, 	     90,0,66,96 },
	 { "Perc#1",	    0,10,P2u_Percussion1,   100,0,24,96 },
	 { "Violins",       0,11,P2u_ArcoViolins,    90,0,55,96 },
	 { "Tremolo",       0,12,P2u_Tremolo2,       80,0,24,96 },
	 { "Harmonics",	    0,13,P2u_StrHarmonic1,   90,0,66,84 },
	 { "ChamberWind",   0,14,P2u_ChamberWind,   100,0,30,80 },
	 { "Epilogue",	    0,15,P2p_Epilogue,	    100,0,30,80 },

	 { "GrandPiano",    1, 0,P1u_GrandPiano,    100,0,24,96 },
	 { "GrandPiano",    1, 1,P1u_GrandPiano,    100,0,24,96 },
	 { "GrandPiano",    1, 2,P1u_GrandPiano,    100,0,24,96 },
	 { "GrandPiano",    1, 3,P1u_GrandPiano,    100,0,24,96 },
	 { "Piano3",	    1, 4,P1u_Clavichord,     90,0,24,96 },
	 { "Piano4",	    1, 5,P1u_PianoHarp,      90,0,24,96 },
	 { "Harp#3",	    1, 6,P1u_ModernHarp,     90,0,24,96 },
	 { "Marimba",	    1, 7,P1u_Marimba,       100,0,48,72 },
	 { "Vibraphone",    1, 8,P1u_Vibraphone,    100,0,48,72 },
	 { "Perc#2",	    1, 9,P1u_LatinPerc,      90,0,24,96 },
	 { "Perc#3",        1,10,P1u_FullPerc,       90,0,24,96 },
	 { "Horn3",	    1,11,P1u_FrenchHorn,     90,0,48,72 },
	 { "Glock",	    1,12,P1u_Glockenspiel,  100,0,66,96 },
	 { "Perc#3",	    1,13,P1u_LatinDrums,     90,0,40,72 },
	 { "Orch#2",        1,14,P1u_OrchestraHit,   80,0,36,54 },
	 { "Strings1",	    1,15,P1u_StaccStrings,  100,0,64,96 },

	 { "",		    2, 0,0,  		    100,0,24,96 },
	 { "",		    2, 1,0,  		    100,0,24,96 },
	 { "",		    2, 2,0,  		    100,0,24,96 },
	 { "",		    2, 3,0,  		    100,0,24,96 },
	 { "",		    2, 4,0,  		    100,0,24,96 },
	 { "",		    2, 5,0,  		    100,0,24,96 },
	 { "",		    2, 6,0,  		    100,0,24,96 },
	 { "",		    2, 7,0,  		    100,0,24,96 },
	 { "",		    2, 8,0,  		    100,0,24,96 },
	 { "",		    2, 9,0,  		    100,0,24,96 },
	 { "",		    2,10,0,  		    100,0,24,96 },
	 { "",		    2,11,0,  		    100,0,24,96 },
	 { "",		    2,12,0,  		    100,0,24,96 },
	 { "",		    2,13,0,  		    100,0,24,96 },
	 { "",		    2,14,0,  		    100,0,24,96 },
	 { "",		    2,15,0,  		    100,0,24,96 },

	 { "",		    3, 0,0,  		    100,0,24,96 },
	 { "",		    3, 1,0,  		    100,0,24,96 },
	 { "",		    3, 2,0,  		    100,0,24,96 },
	 { "",		    3, 3,0,  		    100,0,24,96 },
	 { "",		    3, 4,0,  		    100,0,24,96 },
	 { "",		    3, 5,0,  		    100,0,24,96 },
	 { "",		    3, 6,0,  		    100,0,24,96 },
	 { "",		    3, 7,0,  		    100,0,24,96 },
	 { "",		    3, 8,0,  		    100,0,24,96 },
	 { "",		    3, 9,0,  		    100,0,24,96 },
	 { "",		    3,10,0,  		    100,0,24,96 },
	 { "",		    3,11,0,  		    100,0,24,96 },
	 { "",		    3,12,0,  		    100,0,24,96 },
	 { "",		    3,13,0,  		    100,0,24,96 },
	 { "",		    3,14,0,  		    100,0,24,96 },
	 { "",		    3,15,0,  		    100,0,24,96 },
	},
	{ 0,0,0,0,0,0,0,0,0,0,0,0,
	  1,0,0,0,0,1,0,0,1,0,0,0,
	  1,0,0,0,0,0,1,1,0,0,0,0,
	  1,0,0,0,0,0,1,1,0,0,0,0,
	  1,0,0,0,0,1,0,0,1,0,0,0,
	  1,0,0,0,1,0,0,0,0,1,0,0,
	  1,0,0,1,0,0,0,0,0,0,1,0,
	  1,0,1,0,0,0,0,0,0,0,0,1,
	  1,1,0,0,0,0,0,0,0,0,0,0,
	  1,0,1,0,0,0,0,0,0,0,0,0,
	  1,0,0,1,0,0,0,0 }
};
 
/* ==================================================================== */


unsigned char playstat[128*VOICES];
unsigned char scoreorder[VOICES];

extern int Link;
int Linkmem=0;
int hinput=1;
int houtput=1;
int player=0;
int record=1;
int hdump=0;
int ddump=0;
int miditype=0;
int IsTerm=1;
int IsAuto=0;
int MaxiVol=0;
int usetrk=0;

struct termios termt;

#ifndef DTC3
extern __inline__ int getdata(void);
extern __inline__ int putdata(int n);
extern __inline__ int getdat2(void);
extern __inline__ int putdat2(int n);
#if VOICES>32
extern __inline__ int getdat3(void);
extern __inline__ int putdat3(int n);
extern __inline__ int getdat4(void);
extern __inline__ int putdat4(int n);
#endif
#endif
extern void reset_root(void);

#define TRK_BUF_SIZE	262144

unsigned char *Tracks[VOICES];
unsigned char *Tkptr[VOICES];
unsigned char ChXlat[VOICES];

char txtkeyp[] = { 0x08,0xff,0x01,
		   0x05,'k','e','y',' ',0 };

unsigned autolist[256];
unsigned autoexec[256];

#ifdef DTC3
static char C1cmap[16] =
	{ 0x00,0x01,0x02,0x03,0x04,0x05,0x06,0x07,
	  0x08,0x09,0x0a,0x0b,0x0c,0x0d,0x0e,0x0f };
static char C2cmap[16] =
	{ 0x10,0x11,0x12,0x13,0x14,0x15,0x16,0x17,
	  0x18,0x19,0x1a,0x1b,0x1c,0x1d,0x1e,0x1f };
static char C3cmap[16] =
	{ 0x20,0x21,0x22,0x23,0x24,0x25,0x26,0x29,
	  0xff,0x29,0xff,0xff,0xff,0xff,0xff,0xff };
static char C4cmap[16] =
	{ 0x30,0x31,0x32,0x33,0x34,0x35,0x36,0x39,
	  0xff,0x39,0xff,0xff,0xff,0xff,0xff,0xff };
#endif

/* ==================================================================== */


void fexit(int s)
{
  int i;

	reset_root();
	close(Link);

	if (player) {
		usleep(100000);
#ifdef DTC3
		if (fM>0) {
			ioctl(fM, DTC3_MIDIIRQ, 0);
			close(fM);
		}
#ifdef TIMRIRQ
		if (fT>0) {
			ioctl(fT, DTC3_TIMIRQ, 0);
			close(fT);
		}
#endif
		usleep(100000);
		if (fA[0]>0) {
			ioctl(fA[0], DTC3_MPU_HACK, 0);
			ioctl(fA[0], DTC3_MPU_ALLNOTESOFF, 16);
		}
		if (fA[1]>0) {
			ioctl(fA[1], DTC3_MPU_HACK, 0);
			ioctl(fA[1], DTC3_MPU_ALLNOTESOFF, 16);
		}
		if (fA[2]>0) {
			ioctl(fA[2], DTC3_MPU_HACK, 0);
			ioctl(fA[2], DTC3_MPU_ALLNOTESOFF, 8);
		}
		if (fA[3]>0) {
			ioctl(fA[3], DTC3_MPU_HACK, 0);
			ioctl(fA[3], DTC3_MPU_ALLNOTESOFF, 8);
		}
		for (i=0; i<4; i++) if (fA[i]>0) close(fA[i]);
#else		/* DTC3 */
#ifdef MPU1
		all_notes_off(0);
#endif
#ifdef MPU2
		all_notes_off(1);
#endif
#ifdef MPU3
		all_notes_off(2);
#endif
#ifdef MPU4
		all_notes_off(3);
#endif
		usleep(100000);
#ifdef MPU1
		if (ioperm(MPU1,2,0)) exit(1);
#endif
#ifdef MPU2
		if (ioperm(MPU2,2,0)) exit(1);
#endif
#ifdef MPU3
		if (ioperm(MPU3,2,0)) exit(1);
#endif
#ifdef MPU4
		if (ioperm(MPU4,2,0)) exit(1);
#endif
#endif		/* DTC3 */
	}

	if (hinput && IsTerm)
	  if (tcsetattr(STDIN_FILENO, TCSAFLUSH, &termt)<0) exit(1);
	printf("\n");
	if (hdump) {
		if (IsTerm) printf("Saving log file...\n");
		trkdump(logname);
	}
	if (miditype) {
		if (IsTerm) printf("Saving MIDI file...\n");
		mididump(midname);
	}
	if (ddump) {
		if (IsTerm) printf("Saving raw dump file...\n");
		rawdump(dmpname,"transp.dmp");
	}
	if (usetrk) {
		for (i=0; i<VOICES; i++) if (Tracks[i]) free(Tracks[i]);
	}
	exit(s);
}


int serve (void)
{
  FILE *fd;
  char c;
  int i,l,mpu,mid,zs,zt,xmit,xchan;
  int escstate=0;
  unsigned long t=0;		/* time in us of last event */
  static unsigned char buf[64];
  unsigned char *p,*s;

	zs = zt = 0;

	if (ddump)
	  fd = fopen(dmpname,"w");
#ifdef LINKIRQ
	if (ioctl(Link,LINKSETMODE,LINK_MODE_IRQ)<0) return(1);
#endif
	for (;;) {
#if defined(DTC3) && defined(TIMRIRQ)
		read(fT, &c, 1);	/* go sleep on timer */
#endif
#ifdef DTC2
		if (zs) {
			lseek(Linkmem,0,0);
			read(Linkmem,&zt,1);
			if ((zt&0x80)==0) {
				zs=0;
				if (houtput) {
					putchar((zt&0x3f)+' ');
					fflush(stdout);
				}
			}
		}
#endif

		if ((l=ioctl(Link,LINKFASTREAD,0)) >= 0) {
#ifdef DEBUG
			fprintf(stderr,"%02x\n",l);
#endif
			if (l==0) return(0);
			if (ddump) {
			  read (Link, buf, l);
			  fwrite (buf, 1, l, fd);
			}
			else {
#if VOICES>32
			  mpu = ((l&0xC0)>>6);
			  mid = ((l&0x20)!=0);
#else
			  mpu = ((l&0x80)!=0);
			  mid = ((l&0x40)!=0);
#endif
			  l &= 31;
			  read (Link, buf, (mid? l+4:l));
			}
#ifdef DEBUG
	for (i=0; i<(mid? l+4:l); i++) fprintf(stderr," %02x",buf[i]);
	fprintf(stderr,"\n");
#endif
/* -----------------------------------------------------------------------
   MIDI event (from Tn) in buf:
     --			length
     byte 0..l-1	MIDI data
     byte l..l+3	time in usec (opt)
   ----------------------------------------------------------------------- */
			xmit=1;
   			if (mid) memcpy(&t,buf+l,4);

			xchan = ChXlat[(*buf&15)|(mpu<<4)];
			mpu = (xchan&0x30) >> 4;

			if (player && (*buf!=0xff)) {
			  s = playstat+(xchan<<7)+buf[1];
			  i=0;
			  switch(*buf&0xf0) {
			    case 0x80:	if (*s) {
			  		  *s=0;
			  		  buf[2] &= 0x7f;
			  		  goto putmpu;
			    		}
			    		else xmit=0;
			    		break;
			    case 0x90:	if (*s==0) {
			  		  *s=1;
			  		  buf[2] &= 0x7f;
			  		  goto putmpu;
			    		}
			    		else xmit=0;
			    		break;
			    case 0xf0:	
#ifdef DTC3
					write(fA[mpu], buf, 1);
#else		/* DTC3 */
#ifndef DTC2
#if VOICES>32
					switch (mpu) {
					  case 0: putdata(*buf); break;
					  case 1: putdat2(*buf); break;
					  case 2: putdat3(*buf); break;
					  case 3: putdat4(*buf); break;
					}
#else
			    		if (mpu) putdat2(*buf); else putdata(*buf);
#endif
#endif
#endif		/* DTC3 */
			    		i=2;
			    default:
putmpu:			    

#ifdef DTC3
					write(fA[mpu], buf+i, l-i);
#else		/* DTC3 */
#ifndef DTC2
			    		for (; i<l; i++) {
#if VOICES>32
					  switch (mpu) {
					    case 0: putdata(buf[i]); break;
					    case 1: putdat2(buf[i]); break;
					    case 2: putdat3(buf[i]); break;
					    case 3: putdat4(buf[i]); break;
					  }
#else
			  		  if (mpu) putdat2(buf[i]); else putdata(buf[i]);
#endif
			    		}
#endif
#endif		/* DTC3 */
			    		break;
			  }
			}
			if (mid && record && xmit) {
			  switch (*buf) {
			    case 0xff:	i=mpu<<4; goto putmf0;
			    case 0xf0:	if ((buf[1]==0xd)&&(buf[2]==0x18))
			    		  { i=buf[10]|(mpu<<4); goto putmf0; }
			    default:	i = xchan;
/* -----------------------------------------------------------------------
   MIDI event in track buffer:
     byte 0..3		time in usec
     byte 4		length
     byte 5..l+4	MIDI data
   ----------------------------------------------------------------------- */
putmf0:					if ((mpu<4)&&usetrk) {
					  p = Tkptr[i];
					  memcpy(p,&t,4);
					  p[4]=l;
					  memcpy(p+5,buf,l);
					  Tkptr[i] += (l+5);
					  if ((Tkptr[i]-Tracks[i])>=TRK_BUF_SIZE-128)
					    fexit(1);
					}
					break;
			  }
			}
			if (houtput) {
				putchar('*'); fflush(stdout);
			}
		}
		else if (IsTerm && read(STDIN_FILENO, &c, 1)) {
			if (c==27) {
				if (escstate++) {
#ifdef DTC2
				  lseek(Linkmem,0,0);
				  write(Linkmem,&c,1);
#else
				  while (ioctl(Link,LINKFASTWRITE,c) < 0);
#endif
				  return;
				}
			}
			else if (c>' ' && c<127) {
				c |= 0x80;
#ifdef DTC2
				lseek(Linkmem,0,0);
				write(Linkmem,&c,1);
#else
				while (ioctl(Link,LINKFASTWRITE,c) < 0);
#endif
				zs = c;
				if (houtput) {
					putchar(c&0x7f); fflush(stdout);
				}
				if (record&&usetrk) {
					p=Tkptr[0];
					memcpy(p,&t,4); p+=4;
					for (i=0; i<8; i++) *p++=txtkeyp[i];
					*p++=c;
					Tkptr[0]=p;
				}
				escstate=0;
			}
		}
	}
#ifdef LINKIRQ
	if (ioctl(Link,LINKSETMODE,LINK_MODE_POLL)<0) return(1);
#endif
	if (ddump) fclose(fd);
	return 0;
}


int initFm (void)
{
  int e,oflags;
  struct termios buf;

	IsTerm = isatty(STDIN_FILENO);
	if (hinput && IsTerm) {
		if (tcgetattr(STDIN_FILENO,&termt)<0) return(-1);
		buf = termt;
		buf.c_lflag &= ~(ECHO | ICANON);
		buf.c_cc[VMIN] = 0;
		buf.c_cc[VTIME] = 0;
		if (tcsetattr(STDIN_FILENO, TCSAFLUSH, &buf)<0) return(-1);
	}
	usleep(1000);

#ifdef DEBUG
	for (e=0; e<sizeof(Params); e++) fprintf(stderr," %02x",
		((char *)&Params)[e]);
	fprintf(stderr,"\n");
#endif
	fprintf(stderr, "Writing params (%d bytes)\n",sizeof(Params));
	write(Link,(char *)&Params,sizeof(Params));
	fprintf(stderr, "Parameters written\n");
	
	return (0);
}


void params(int argc, char **argv)
{
  int i,np,v;
  char *p;
  double d;

	np=0;
	*insname=0; *pname=0; autolist[0]=0;

	while ((argc--)>0) {
		p = *argv++;
		
		if ((*p=='-') && isalpha(p[1])) {
		  switch(p[1]) {
		    case 'a':	IsAuto  =(p[2]!='-'); break;
		    case 'p':	player	=(p[2]!='-'); break;
		    case 'r':   record	=(p[2]!='-'); break;
		    case 'd':	hdump	=(p[2]!='-');
		    		if (hdump) { usetrk=1; Params.xp[0] |= 1; }
		    		break;
		    case 'm':	miditype=(p[2]!='-');
		    		if (miditype) { usetrk=1; Params.xp[0] |= 1; }
		    		break;
		    case 'v':	if (IsTerm) houtput=(p[2]!='-'); break;
		    case 'b':	p = *argv++; argc--;
		    		strncpy(fname,p,254);
		    		break;
		    case 'f':	p = *argv++; argc--;
		    		strncpy(pname,p,254);
				if (*pname) readparmfile(pname);
		    		break;
		    case 'h':	MaxiVol = 1; break;
		    case 'i':	p = *argv++; argc--;
		    		strncpy(insname,p,254);
				if (*insname) readinsfile(insname);
		    		break;
		    case 'o':	p = *argv++; argc--;
		    		strncpy(logname,p,248); strcat(logname,".log");
		    		strncpy(midname,p,248); strcat(midname,".mid");
		    		break;
		    case 'D':	ddump = 1;
		    		break;
		    case 'F':	if (isdigit(p[2])) {
		    		  i = (p[2]-'0');
		    		  if (i<4) {
		    		  	p = *argv++; argc--;
		  			if (sscanf(p,"%lf",&d)==1) Params.fp[i]=d;
		    		  }
		    		}
		    		break;
		    case 'I':	if (isdigit(p[2])) {
		    		  i = (p[2]-'0');
		    		  if (i<4) {
		    		  	p = *argv++; argc--;
		    		  	if ((*p=='0') && (p[1]=='x')) {
		  			  if (sscanf(p+2,"%x",&v)==1) Params.ip[i]=v;
		    		  	}
		    		  	else {
		  			  if (sscanf(p,"%d",&v)==1) Params.ip[i]=v;
		  			}
		    		  }
		    		}
		    		break;
		    case 'X':	if (isdigit(p[2])) {
		    		  i = (p[2]-'0');
		    		  if (i<8) {
		    		  	p = *argv++; argc--;
		    		  	if ((*p=='0') && (p[1]=='x')) {
		  			  if (sscanf(p+2,"%x",&v)==1) Params.xp[i]=v;
		    		  	}
		    		  	else {
		  			  if (sscanf(p,"%d",&v)==1) Params.xp[i]=v;
		  			}
		    		  }
		    		}
		    		break;
		  }
		}
		else {
		  if (np<4)
		  	if (sscanf(p,"%lf",&d)==1) { Params.fp[np]=d; np++; }
		  else if (np<8)
		  	if (sscanf(p,"%d",&v)==1) { Params.ip[np-4]=v; np++; }
		  else if (np<16)
		  	if (sscanf(p,"%d",&v)==1) { Params.xp[np-8]=v; np++; }
		}
	}

	if (*logname==0) {
		strcpy(logname,tmpnam(NULL)); strcpy(midname,logname);
		strcat(logname,".log"); strcat(midname,".mid");
	}
	if (autolist[0]==0) IsAuto=0;

	if (ddump) {
	  hdump = miditype = record = player = 0;
	  Params.xp[0] = (Params.xp[0])&(~3) | 2;
	}
}

/* ==================================================================== */


int main (int argc,char **argv)
{
  FILE *ft;
  int i,e;
  pid_t pid;
 
	memset(playstat,0,128*VOICES);
	for (i=0; i<VOICES; i++) scoreorder[i]=ChXlat[i]=i;
	params(argc-1,argv+1);

	for (i=0; i<VOICES; i++) {
	  if (usetrk) {
		Tracks[i] = (unsigned char *)malloc(TRK_BUF_SIZE);
		Tkptr[i] = Tracks[i];
		if (Tracks[i]==NULL) {
			perror("Track alloc"); exit(1);
		}
	  }
	  else {
		Tkptr[i] = Tracks[i] = NULL;
	  }
	}
	
	if ((ft=fopen(fname,"r"))==NULL) {
		errno=ENOENT; perror(fname); exit(1);
	}
	fclose(ft);

	if (e=root_init(fname)) {
		printf("Transputer error %d\n",e); exit(1);
	}

	if (player) {
#ifdef DTC3
		fM = open("/dev/dtc3", O_RDWR);
#ifdef TIMRIRQ
		fT = open("/dev/dtc3tim", O_RDONLY);
		ioctl(fT, DTC3_TIMFSEL, 2);
		ioctl(fT, DTC3_TIMEN, 1);
#endif
		fA[0] = open("/dev/dtc3mid1", O_WRONLY);
		fA[1] = open("/dev/dtc3mid2", O_WRONLY);
		fA[2] = open("/dev/dtc3mid3", O_WRONLY);
		fA[3] = open("/dev/dtc3mid4", O_WRONLY);
		if ((fA[0] < 0) || (fA[1] < 0) || (fA[2] < 0) || (fA[3] < 0)) {
			perror("MIDI device");
			exit(1);
		}
		ioctl(fA[0], DTC3_MPU_HACK, MPU_PROTEUSHACK);
		ioctl(fA[1], DTC3_MPU_HACK, MPU_PROTEUSHACK);
		ioctl(fA[2], DTC3_MPU_HACK, MPU_ROLANDHACK);
		ioctl(fA[3], DTC3_MPU_HACK, MPU_ROLANDHACK);

		ioctl(fA[0], DTC3_MPU_CHANMAP, C1cmap);
		ioctl(fA[1], DTC3_MPU_CHANMAP, C2cmap);
		ioctl(fA[2], DTC3_MPU_CHANMAP, C3cmap);
		ioctl(fA[3], DTC3_MPU_CHANMAP, C4cmap);

		ioctl(fA[0], DTC3_MPU_ALLNOTESOFF, 16);
		ioctl(fA[1], DTC3_MPU_ALLNOTESOFF, 16);
		ioctl(fA[2], DTC3_MPU_ALLNOTESOFF, 8);
		ioctl(fA[3], DTC3_MPU_ALLNOTESOFF, 8);

#ifdef TIMRIRQ
		ioctl(fT, DTC3_TIMIRQ, 1);
#endif
#ifdef MIDIIRQ
		ioctl(fM, DTC3_MIDIIRQ, 1);
#endif
#else		/* DTC3 */
#ifdef MPU1
		if (ioperm(MPU1,2,1)) exit(1);
#endif
#ifdef MPU2
		if (ioperm(MPU2,2,1)) exit(1);
#endif
#ifdef MPU3
		if (ioperm(MPU3,2,1)) exit(1);
#endif
#ifdef MPU4
		if (ioperm(MPU4,2,1)) exit(1);
#endif
#ifdef MPU1
		all_notes_off(0);
#endif
#ifdef MPU2
		all_notes_off(1);
#endif
#ifdef MPU3
		all_notes_off(2);
#endif
#ifdef MPU4
		all_notes_off(3);
#endif
		usleep(100000);
#if !defined(DTC2)&&!defined(DTC3)
#ifdef MPU1
		repeat_cmd401(0,UART);
#endif
#ifdef MPU2
		repeat_cmd401(1,UART);
#endif
#ifdef MPU3
		repeat_cmd401(2,UART);
#endif
#ifdef MPU4
		repeat_cmd401(3,UART);
#endif
#endif
#endif		/* DTC3 */
	}

	if (!initFm()) {

#ifdef DTC2
		Linkmem = open("/dev/linkmem", O_WRONLY);
		if (Linkmem <= 0) goto quit;
#endif
		signal(SIGINT,fexit);

		if ((pid = fork()) < 0) {
			printf("Fork error\n");
			fexit(1);
		}
		else if (pid!=0) {	/* parent */
			usleep(100000);
			while (ioctl(Link,LINKFASTWRITE,1) < 0);
			serve();
		}
		else {			/* child */
		 int sv,c,autocnt=0;
		  if (IsAuto) {
		  	while (sv=autolist[autocnt]) {
		  		sleep(sv);
		  		if (c=autoexec[autocnt]) {
#ifdef DTC2
					lseek(Linkmem,0,0);
					write(Linkmem,&c,1);
#else
					while (ioctl(Link,LINKFASTWRITE,c) < 0);
#endif
					if (houtput) {
					   putchar('|'); fflush(stdout);
					}
		  		}
		  		autocnt++;
		  	}
	  	  	if (houtput) printf("[EC]");
	  	  	kill(getppid(),SIGINT);
	  	  }
		  _exit(0);
		}
#ifdef DTC2
		close(Linkmem);
quit:
#endif
		printf("[EP]");
		fexit(0);
	}
	else {
		printf("Init error %d\n",e); exit(1);
	}
	printf("\n** we should not be here **\n");
#ifdef DTC3
	if (fM>0) {
		ioctl(fM, DTC3_MIDIIRQ, 0);
		close(fM);
	}
#ifdef TIMRIRQ
	if (fT>0) {
		ioctl(fT, DTC3_TIMIRQ, 0);
		close(fT);
	}
#endif
	usleep(100000);
	if (fA[0]>0) ioctl(fA[0], DTC3_MPU_ALLNOTESOFF, 16);
	if (fA[1]>0) ioctl(fA[1], DTC3_MPU_ALLNOTESOFF, 16);
	if (fA[2]>0) ioctl(fA[2], DTC3_MPU_ALLNOTESOFF, 8);
	if (fA[3]>0) ioctl(fA[3], DTC3_MPU_ALLNOTESOFF, 8);
	for (i=0; i<4; i++) close(fA[i]);
#endif	
}

