
/* ==================================================================== */
/*			READ PARAMS module				*/
/* ==================================================================== */

/* Included files */

#include <unistd.h>
#include <fcntl.h>
#include <sys/stat.h>
#include <stdio.h>
#include <errno.h>
#include <ctype.h>
#include <string.h>
#include <math.h>
#include <signal.h>
#include <termios.h>
#include <linux/link.h>
#include <asm/io.h>

#include "../proteus.h"
#include "../canvas.h"
#include "../fm_def.h"
#include "mpu401.h"

extern char fname[];
extern char insname[];
extern _paramblk Params;
extern unsigned char playstat[],scoreorder[];
extern int Link;
extern int miditype;
extern int MaxiVol;
extern int gmpu;
extern unsigned char *Tracks[];
extern unsigned char *Tkptr[];
extern unsigned char ChXlat[];
extern unsigned autolist[],autoexec[];

int Mxv = 0;

/* ==================================================================== */


static void scanins(int ns,char *s)
{
  Instr *I;
  char name[32];
  char *p;
  int i,mpu,chan;
  long v;

	while (*s!='"') s++;
	p=strtok(s,"\n\t\""); strncpy(name,p,30);
	p=strtok(NULL,"\n\t ,"); if (sscanf(p,"%ld",&v)==1) mpu=v&3;
	p=strtok(NULL,"\n\t ,"); if (sscanf(p,"%ld",&v)==1) chan=v&15;
	i = mpu*16+chan;

	if (i < VOICES) {
		I = &(Params.instr[i]);
		while (*s!='"') s++;
		strcpy(I->name,name);
		I->mpu=mpu; I->chan=chan;
		p=strtok(NULL,"\n\t ,"); if (sscanf(p,"%ld",&v)==1) I->patch=v;
		p=strtok(NULL,"\n\t ,");
		if (sscanf(p,"%ld",&v)==1) {
			I->volume=v;
			if (v > Mxv) Mxv = v;
		}
		p=strtok(NULL,"\n\t ,"); if (sscanf(p,"%lx",&v)==1) I->flags=v;
		p=strtok(NULL,"\n\t ,"); if (sscanf(p,"%ld",&v)==1) I->rmin=v;
		p=strtok(NULL,"\n\t ,"); if (sscanf(p,"%ld",&v)==1) I->rmax=v;
		if (ns>0) scoreorder[ns-1]=i;
	}
}


static void maxvins(void)
{
  Instr *I;
  float df;
  int i;

	if (!MaxiVol) return;
	df = 120.0 / Mxv;
	if (df < 1.001) return;
	printf("Increase vol by %f\n",df);

	for (i=0; i<VOICES; i++) {
		I = &(Params.instr[i]);
		I->volume *= df;
	}
}


/* ==================================================================== */

static char cppfn[] = "/lib/cpp ";


int readparmfile(char *fnam)
{
  FILE *fp;
  char Line[512];
  char *p,*s;
  int i,j,n,v;
  double d;
  char pfn[256] = "";
  int autocnt=0;

	Mxv = 0;
	strcpy(pfn,cppfn);
	strcat(pfn,fnam);
	printf("Exec: [%s]\n",pfn);

	if ((fp=popen(pfn,"r"))==NULL) return(1);

	while (fgets(Line,254,fp)!=NULL) {
		s=Line;
		i=strlen(s)-1; if (s[i]=='\n') s[i]=0;
		while (isspace(*s)) s++;
		if (*s!='%') continue;
		s++;
		if (!strncmp("GEN",s,3)) {
			s+=3; while (isspace(*s)) s++;
			strncpy(fname,s,254);
		}
		else if (!strncmp("MOD2",s,4)) {
			i=0;
			while (i<128) {
			  if (fgets(Line,254,fp)==NULL) return;
			  s=Line;
			  while (*s) {
			    if (isspace(*s)) { s++; continue; }
			    if (isxdigit(*s)) {
			    	char c = toupper(*s++) - '0';
			    	Params.Mtempl[i++] = (c>9? c-7:c);
			    	if (i>127) break;
			    }
			    else {
			    	Params.Mtempl[i++] = 0; s++;
			    }
			  }
			}
		}
		else if (!strncmp("MOD",s,3)) {
			i=0;
			while (i<128) {
			  if (fgets(Line,254,fp)==NULL) return;
			  s=Line;
			  while (*s) {
			    if (isspace(*s)) { s++; continue; }
			    if (isdigit(*s)) {
			    	Params.Mtempl[i++] = (*s++)-'0';
			    	if (i>127) break;
			    }
			    else {
			    	Params.Mtempl[i++] = 0; s++;
			    }
			  }
			}
		}
		else if (!strncmp("INS",s,3)) {
			s+=3;
			if (sscanf(s,"%d",&v)==1) {
				while (isdigit(*s)) s++; while (isspace(*s)) s++;
				scanins(v,s);
			}
		}
		else if (!strncmp("CMAP",s,4)) {
			s+=4;
			while (isspace(*s)) s++;
			if (sscanf(s,"%d",&i)==1) {
			  while (isdigit(*s)) s++; while (isspace(*s)) s++;
			  if (sscanf(s,"%d",&v)==1) {
				ChXlat[i&0x3f] = v&0x3f;
			  }
			}
		}
		else if (!strncmp("AUT",s,3)) {
			s+=3; while (!isspace(*s)) s++;
			p=strtok(s,"\n\t ,");
			if ((sscanf(p,"%d",&v)==1)&&(autocnt<255)) {
			  autolist[autocnt]=v;
			  p=strtok(NULL,"\n\t ,");
			  autoexec[autocnt]=(isalnum(*p)? *p:0);
			  autolist[++autocnt]=0;
			}
		}
		else {
			switch(toupper(*s)) {
			  case 'F': if (isdigit(s[1])) {
			  		i=s[1]-'0'; if (i>3) break;
			  		s+=2; while (isspace(*s)) s++;
			  		if (sscanf(s,"%lf",&d)==1)
			  			Params.fp[i]=d;
			  	    }
			  	    break;
			  case 'I': if (isdigit(s[1])) {
			  		i=s[1]-'0'; if (i>3) break;
			  		s+=2; while (isspace(*s)) s++;
			  		if ((*s=='0')&&(s[1]=='x')) {
			  		  if (sscanf(s+2,"%x",&v)==1) Params.ip[i]=v;
			  		}
			  		else {
			  		  if (sscanf(s,"%d",&v)==1) Params.ip[i]=v;
			  		}
			  	    }
			  	    break;
			  case 'X': if (isdigit(s[1])) {
			  		i=s[1]-'0'; if (i>7) break;
			  		s+=2; while (isspace(*s)) s++;
			  		if ((*s=='0')&&(s[1]=='x')) {
			  		  if (sscanf(s+2,"%x",&v)==1) Params.xp[i]=v;
			  		}
			  		else {
			  		  if (sscanf(s,"%d",&v)==1) Params.xp[i]=v;
			  		}
			  	    }
			  	    break;
			}
		}
	}
	pclose(fp);
	maxvins();
	return(0);
}


int readinsfile(char *inam)
{
  FILE *fp;
  char Line[512];
  char *s;
  int i,j,n,v;
  double d;
  char pfn[256] = "";

	Mxv = 0;
	strcpy(pfn,cppfn);
	strcat(pfn,inam);
	printf("Exec: [%s]\n",pfn);

	if ((fp=popen(pfn,"r"))==NULL) return(1);

	while (fgets(Line,254,fp)!=NULL) {
		s=Line;
		i=strlen(s)-1; if (s[i]=='\n') s[i]=0;
		while (isspace(*s)) s++;
		if (*s!='%') continue;
		s++;
		if (!strncmp("INS",s,3)) {
			s+=3;
			if (sscanf(s,"%d",&v)==1) {
				while (isdigit(*s)) s++; while (isspace(*s)) s++;
				scanins(v,s);
			}
		}
	}
	pclose(fp);
	maxvins();
	return(0);
}


/* ==================================================================== */

