
/* ================================================================== */
/*				  ICC.H                               */
/*			    compatibility box                         */
/* ================================================================== */

#ifndef _miccc_h_
#define _miccc_h_	1

#define TRANSP16B       (_PTYPE=='2'||_PTYPE=='3')
#define TRANSP32B       (_PTYPE!='2'&&_PTYPE!='3')
#define TRANSPFLT       (_PTYPE=='8'||_PTYPE=='9')

#if TRANSP16B
#error This is the T4/8 version
#endif

#ifndef _ICC
#error Needs INMOS ANSI C
#endif

#define asm		__asm

#pragma IMS_modpatchsize(2)
#pragma IMS_codepatchsize(5)

/* ================================================================== */

#define WORDSIZE	4
#define WORDSHIFT	2
#define WORDMASK	0xfffffffc
#define WORDBITS	3
#define DWORDMASK	0xfffffff8
#define MinInt		0x80000000
#define Lo16Mask	0xffff
#define Hi16Mask	0xffff0000

/* ================================================================== */

#define BYTES(n)	((n)<<WORDSHIFT)
#define WORDS(n)	((n)>>WORDSHIFT)
#define MEMORY(n)	(BYTES(n)^MinInt)
#define STDALLOC	BYTES(255)
#define SAVEAREA	((struct _SaveArea *)MEMORY(11))
#define SAVE_WDESC	((int *)MEMORY(11))
#define SAVE_IPTR 	((int *)MEMORY(12))
#define SAVE_AREG 	((int *)MEMORY(13))
#define SAVE_BREG 	((int *)MEMORY(14))
#define SAVE_CREG 	((int *)MEMORY(15))
#define SAVE_STATUS 	((int *)MEMORY(16))
#define SAVE_Ereg   	((int *)MEMORY(17))

/* ================ TYPEDEFS ================= */

typedef int int32;
typedef int WORD;
typedef short int16;
typedef unsigned int UWORD;
typedef unsigned int UINT;
typedef unsigned long ULONG;
typedef unsigned short USHORT;

#define BYTE0(i)	*((char *)&(i))
#define BYTE1(i)	((char *)&(i))[1]
#define BYTE2(i)	((char *)&(i))[2]
#define BYTE3(i)	((char *)&(i))[3]
#define WORD0(s)	*((int *)s)
#define WORD1(s)	((int *)s)[1]

typedef struct { int l,m,h; } xdouble;

#ifndef NULL
#define NULL	((void *)0)
#endif

#ifndef _CHAN_H
#define _CHAN_H 1
typedef	void *CHAN;
#endif
#ifndef _channel_h
typedef void *Channel;
#endif

/* ================================================================== */

#define T_LINKS		4

/* Address for physical links on transputers */

#define	Link0Output	((CHAN *)MEMORY(0))
#define Link1Output	((CHAN *)MEMORY(1))
#define Link2Output	((CHAN *)MEMORY(2))
#define Link3Output	((CHAN *)MEMORY(3))
#define	Link0Input	((CHAN *)MEMORY(4))
#define	Link1Input	((CHAN *)MEMORY(5))
#define	Link2Input	((CHAN *)MEMORY(6))
#define	Link3Input	((CHAN *)MEMORY(7))

/* Address of event pin on transputers */

#define EventReq	((CHAN *)MEMORY(8))
#define NotProcess_P    ((void *)MinInt)
#define Not_ProcessP    ((void *)MinInt)
#define NOCHAN		((void *)MinInt)
#define NotWorkspace_P  (MinInt+1)
#define Not_WorkspaceP  (MinInt+1)
#define UNDEFINED	(MinInt)

#define THREAD_NOTURG 1
#define THREAD_URGENT 0

/* ================================================================== */

#pragma IMS_translate(ChanOutTimeFail, "ChanOutTimeFail%c")
#pragma IMS_translate(ChanInTimeFail, "ChanInTimeFail%c")

/* INLINE functions */

#define chan_reset	ChanReset
#define ChanReset(ch)	asm { ld ch; resetch; }
#define chan_init	ChanInit
#define ChanInit(ch)	asm { mint; ld ch; stnl 0; }

#define chan_in_byte(b,ch)	asm { ld b; ld ch; ldc 1; in; }
#define chan_in_word(w,ch)	asm { ld w; ld ch; ldc 1; bcnt; in; }
#define chan_in_long(w,ch)	asm { ld w; ld ch; ldc 4; in; }
#define chan_in_message(l,p,ch)	asm { ld p; ld ch; ld l; in; }
#define _inbyte			chan_in_byte
#define _inword			chan_in_word
#define __inbyte		chan_in_byte
#define __inword		chan_in_word
#define chan_out_byte(b,ch)	asm { ld b; ld ch; ldc 1; out; }
#define chan_out_word(w,ch)	asm { ld w; ld ch; ldc 1; bcnt; out; }
#define chan_out_long(w,ch)	asm { ld w; ld ch; ldc 4; out; }
#define chan_out_message(l,p,ch) asm { ld p; ld ch; ld l; out; }
#define _outbyte(b,ch)		asm { ld ch; ld b; ajw -1; outbyte; ajw 1; }
#define _outword(w,ch)		asm { ld ch; ld w; ajw -1; outword; ajw 1; }
#define __outbyte(b,ch)		asm { ld ch; ld b; outbyte; }
#define __outword(w,ch)		asm { ld ch; ld w; outword; }

#define timer_now(v)		asm { ldtimer; st v; }
#define timer_wait(t)		asm { ld t; tin; }
#define timer_delay(t)		asm { ldtimer; ld t; sum; tin; }
#define thread_deschedule()	asm { ldtimer; tin; }

#define save_Hiqueue(p)		asm { ld p; saveh; }
#define save_Loqueue(p)		asm { ld p; savel; }
#define rest_Loqueue(p)		asm { ld p.Fp; stlf; ld p.Bp; stlb; }
#define clear_Loqueue()		asm { mint; stlf; }

#define _memcpy(dest,src,n)	asm { ldabc n,dest,src; move; }

/* LSW(src) to LSW(dest) */
#define MOVE2LL(d,s)		asm { ldabc 2,&(d),&(s); move; }

/* LSW(src) to MSW(dest) */
#define MOVE2LH(d,s)		asm { ldabc 2,((char *)&(d))+2,&(s); move; }

/* MSW(src) to LSW(dest) */
#define MOVE2HL(d,s)		asm { ldc 0; st d;\
				      ldabc 2,&(d),((char *)&(s))+2; move; }

#define align(x)		asm { ld x; wcnt; rev; eqc 0; eqc 0;\
				      add; bcnt; st x; }

#ifdef DTC2
/* move to/from QRAM
	Creg=source stride
	Breg=dest stride
	Areg=row count		move2dinit
	Creg=source
	Breg=dest
	Areg=row width		move2dall
*/
/*
	move 256 bytes from the lsb-word-aligned quadport RAM at
	page 2 into a 256-byte buffer
*/
#define fromQRAMall(buf)	asm { ldc 4; ldc 1; ldc 256; move2dinit;\
				      ldc 0x400800; ld buf; ldc 1; move2dall; }

#define fromQRAM(a,v)		asm { ldc 4; ldc 1; ldc 4; move2dinit;\
				      ld a; ldc 2; shl; ldc 0x400800; sum;\
				      ld &v; ldc 1; move2dall; }

#define toQRAMall(buf)		asm { ldc 1; ldc 4; ldc 256; move2dinit;\
				      ld buf; ldc 0x400800; ldc 1; move2dall; }

#define toQRAM(a,v)		asm { ldc 1; ldc 4; ldc 4; move2dinit;\
				      ld &v;\
				      ld a; ldc 2; shl; ldc 0x400800; sum;\
				      ldc 1; move2dall; }
#endif

/* ================================================================== */
/*
  undocumented Transputer instructions
*/

#define fpxload		byte 0x28;byte 0xf5
#define fpxstore	byte 0x28;byte 0xf0
#define ldshadow	ldc 0x2c;fpentry
#define stshadow	ldc 0x30;fpentry

/* ================================================================== */

typedef struct {
  int   *Wptr;
  void (*Iptr)(void);
 } _sysbreakv;

#define HiBrk		((_sysbreakv *)MEMORY(0x1C))
#define LoBrk		((_sysbreakv *)MEMORY(0x1E))

#define LOADERPAR	0x80000134	/* INMOS ICC bootstrap data */

/* ================================================================== */
/*
  ICC variables and structures
*/

typedef struct {
  int mem_min;
  int mem_max;
  int n1;		/* points to next word */
  int n2;		/* somewhere over mem_max */
  int n3;		/* somewhere over mem_max */
  int n4;		/* ? */
  int mem_start;
  int bootlink;
 } _sysbootv;

extern int _IMS_heap_base,_IMS_heap_front;
extern int _IMS_stack_base,_IMS_stack_limit;
extern int _IMS_StartTime;
#pragma IMS_translate(errno,"_IMS_errno")
extern volatile int errno;
extern int _IMS_last_recorded_wptr,_IMS_max_wptr_extent;
extern int _IMS_shared_area,_IMS_startenv;
#pragma IMS_translate(first_init_block,"first%init%block")
extern int first_init_block;
#pragma IMS_translate(static_base,"static%base")
#pragma IMS_translate(static_space,"static%space")
#pragma IMS_translate(text_base,"text%base")
extern int static_base,text_base,static_space;

/* ================================================================== */

#define thread_start	_ThreadStart

extern  void thread_start(void (* /*fn*/)(), char * /*ws*/,
			  int /*wssize*/, int /*flags*/,
			  int /*nargs*/, ...);

extern void STOP(void);

/* ================================================================== */

#endif

