
/* =================================================================== */
/*	   			MPLAY.H                                */
/* =================================================================== */

#ifndef T_MPLAY_H
#define T_MPLAY_H 1

#ifdef __GNUC__
#error This is Transputer C code!
#endif

/*#if (_PTYPE=='8'||_PTYPE=='9')
typedef double FLOAT;
#else*/
typedef float FLOAT;
/*#endif*/

#include <stddef.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "icc.h"

#include "../fm_def.h"

/* =================================================================== */

#define BITTIME		32		/* usec */
#define BYTETIME	320		/* usec */

#define QUARTER		500		/* msec */
#define EIGHTH		(QUARTER/2)
#define SIXTEENTH	(QUARTER/4)
#define THIRTYSECOND	(QUARTER/8)
#define DOTQUARTER	(QUARTER*3/2)
#define DOTEIGHTH	(QUARTER*3/4)
#define HALF		(QUARTER*2)
#define WHOLE		(QUARTER*4)

#define NMPUS		(VOICES/16)

/* status byte:
	bits 0-4	len (1-31)
32 voices:
	bit 5		?
	bit 6		? midi flag
	bit 7		MPU
64 voices:
	bit 5		?
	bits 6-7	MPU
*/
#ifdef DTC2
#define UARTBASE	0x400ce0		/* 0x338<<2 */
#define UART1D		*((char *)0x400ce0)
#define UART1S		*((char *)0x400ce4)
#define UART2D		*((char *)0x400ce8)
#define UART2S		*((char *)0x400cec)
#define UART3D		*((char *)0x400cf0)
#define UART3S		*((char *)0x400cf4)
#define UART4D		*((char *)0x400cf8)
#define UART4S		*((char *)0x400cfc)

#define HostCmd		*((char *)0x400800)
#endif

#if VOICES>32
#define NMPUMASK	0x30
#define C2MPU(c)	(((c)&0x30)<<2)		/* bits 4,5 to 6,7 */
#define MIDIBIT		0x20
#else
#define NMPUMASK	0x10
#define C2MPU(c)	(((c)&0x10)<<3)		/* bit 4 to 7 */
#define MIDIBIT		0x40
#endif

#define NLOCK(c)	((c)|0x80)
#define NUNLOCK(c)	((c)|0x40)

/* =================================================================== */


/* simple linked list element */
struct sllist {
  struct sllist *next;
  int data;
 };


/* doubly-linked list header */
struct dllist {
  void *head;
  void *EOL;
  void *tail;
 };

typedef struct dllist HLIST;


struct _SaveSwap {
  int Fp,Bp;
 };


/* floating point coprocessor save register area, optional per task */
struct fpxregs
{
  xdouble xa,xb,xc;
};


struct _SaveArea {
  int WDESCsave;	/* save area, 7 words @8000002C */
  int IPTRsave;
  int AREGsave;
  int BREGsave;
  int CREGsave;
  int STATUS;
  int Ereg;
 };


/* =================================================================== */
/*
	EVENT PROCESS: STACK FRAME
	Ŀ
       0Ĵ-13 W[-14]=next ptr
       1Ĵ-12 W[-13]=data(0)
       2Ĵ-11 W[-12]=data(1)
       3Ĵ-10 W[-11]=data(2)
       4Ĵ -9 W[-10]=data(3)
       5Ĵ -8 W[ -9]=data(4)
       6Ĵ -7 W[ -8]=data(5)
       7Ĵ -6 W[ -7]=data(6)
       8Ĵ -5 W[ -6]	(timer use)
       9Ĵ -4 W[ -5]
      10Ĵ -3 W[ -4]
      11Ĵ -2 W[ -3]
      12Ĵ -1 W[ -2]
      13Ĵ  0 W[ -1]=Iptr  ----> W[0]
      14Ĵ  1 W[  0]=RA
      15Ĵ  2 W[  1]=SL
      16Ĵ  3 W[  2]=time
      17Ĵ  4 W[  3]=&EvQueue
      18Ĵ  5 W[  4]=&EndXQ
      19Ĵ  6 W[  5]=&EvCh;
      20
 */

#define NPARA_E		4
#define NDATA_E		28

struct _event {
  struct _event *next;
  int  flags;
  char data[NDATA_E];
  int  frame[8];
  int  param[NPARA_E];
 };

typedef struct _event EVENT;

#define EVSIZE		(sizeof(EVENT)/sizeof(int))

/* =================================================================== */

typedef struct {

	Instr I;

	CHAN ichan;
	void (*func)();
	char *ws;
} TInstr;

/* =================================================================== */

extern TInstr InstrTab[];
extern _paramblk Params;

#define HOSTP_F(n)	Params.fp[(n)]
#define HOSTP_I(n)	Params.ip[(n)]
#define HOSTP_X(n)	Params.xp[(n)]

#define MINRANGE(n)	InstrTab[(n)].I.rmin
#define MAXRANGE(n)	InstrTab[(n)].I.rmax
#define IFLAGS(n)	InstrTab[(n)].I.flags

#ifndef DTC2
extern int   HostCmd;
#endif
extern unsigned Btempo,TimeD;
extern int   MIDmode,Debug;
extern int   Presets[];
extern unsigned Bittab[];

extern FLOAT Fseq;
extern unsigned Rseed;

int  InstrSetup(void);
void InitGen(void);
void GenLoop(void);

unsigned EOS (unsigned t);
unsigned ntempo (unsigned nt,unsigned t);
unsigned oevent (int p1,int p2,unsigned t);
unsigned mevent (char *p1,int p2,unsigned t);
unsigned idenom (int chan,int num,unsigned t);
unsigned noteon (int chan,int h,int v,unsigned t);
unsigned allnoteoff (int chan,unsigned t);
unsigned noteoff (int chan,int h,int v,unsigned t);
unsigned volume (int chan,int v,unsigned t);
unsigned patch (int chan,int n,unsigned t);
unsigned pressure (int chan,int v,unsigned t);
unsigned chknote (NoteEvent *N,unsigned t);
unsigned note (NoteEvent *N,unsigned t);
unsigned repnote (NoteEvent *N,unsigned iT,int dT,unsigned t);
unsigned fullnote (NoteEvent *N,unsigned t);
unsigned pedal (NoteEvent *N,unsigned t);
unsigned xparam (int chan,int n,int v,unsigned t);
unsigned ocparam (int chan,int n,unsigned t);
unsigned rcparam (int chan,int n,int v,unsigned t);
unsigned nothing (NoteEvent *N,unsigned t);

void  Srand(unsigned seed);
unsigned Rand(void);
FLOAT onefrand (FLOAT lb, FLOAT ub);
void  f1init (FLOAT sequence);
Pair *rmandel(int mn,int mx);
void  mbinit(float r,float i);

/* =================================================================== */

#endif

